@@ -135,7 +135,7 @@ int RD_GetFFLine(FILE *fp, char *line) {
 /*     StringToCode: turn a <=8 byte string into a 64 bit int              */
 /*                                                                         */
 /***************************************************************************/
-static long long RD_StringToCode(char *tmpcode) {
+static long long RD_StringToCode(const char *tmpcode) {
   long long code;
   char *cptr;
   int i;
@@ -143,7 +143,7 @@ static long long RD_StringToCode(char *tmpcode) {
   cptr=(char*) &code;
   code=0;
   
-  for (i=0;i<(int)strlen(tmpcode); i++) {
+  for (i=0; i<(int)strlen(tmpcode); i++) {
     cptr[i]=tmpcode[i];
   }
   for (i=strlen(tmpcode); i<8; i++) {
@@ -320,7 +320,7 @@ static int RD_GetFileFormatIndex(char *filename,int *i_format) {
 /*    returns error code                                                   */
 /*                                                                         */
 /***************************************************************************/
-static int RD_GetFieldIndex(char *field_code,int i_format, int *i_field) {
+static int RD_GetFieldIndex(const char *field_code,int i_format, int *i_field) {
   unsigned long long lfield_code;
   static int i_f=0;
   int n;	
