@@ -23,8 +23,10 @@
 #include <fitsio.h>
 #include <stdlib.h>
 
-LFIIOSource::LFIIOSource( const QString& filename, const QString& type )
-: KstDataSource( filename, type )
+#define TIME_FIELD  "TIME"
+
+LFIIOSource::LFIIOSource( KConfig *cfg, const QString& filename, const QString& type )
+: KstDataSource( cfg, filename, type )
 {
   if( type.isEmpty( ) || type == "LFIIO" )
   {
@@ -41,6 +43,13 @@ LFIIOSource::~LFIIOSource( )
 }
 
 
+bool LFIIOSource::reset()
+{
+  _fieldList.clear();
+  return _valid = initFile();
+}
+
+
 bool LFIIOSource::initFile( )
 {
   KstObject::UpdateType updateType;
@@ -59,7 +68,9 @@ bool LFIIOSource::initFile( )
 
 KstObject::UpdateType LFIIOSource::update( int u )
 {
-  Q_UNUSED( u )
+  if (KstObject::checkUpdateCounter(u)) {
+    return lastUpdateResult();
+  }
 
   KstObject::UpdateType updateType =  KstObject::NO_CHANGE;
   QString               strTemplate;
@@ -79,7 +90,7 @@ KstObject::UpdateType LFIIOSource::update( int u )
   int                   i;
 
   _valid  = false;
-
+  
   if( !_filename.isNull( ) && !_filename.isEmpty( ) )
   {
     iResult = fits_open_table( &ffits, _filename.ascii( ), READONLY, &iStatus );
@@ -98,10 +109,11 @@ KstObject::UpdateType LFIIOSource::update( int u )
           _fieldList.append( "INDEX" );
 
           _valid = true;
+          _bHasTime = false;
 
           //
-	        // need to multiply lNumFrames by the maximum value of the vector repeat value...
-	        //
+          // need to multiply lNumFrames by the maximum value of the vector repeat value...
+          //
           for( i=0; i<iNumCols; i++ )
           {
             iStatus = 0;
@@ -130,8 +142,32 @@ KstObject::UpdateType LFIIOSource::update( int u )
             }
           }
 
+          //
+          // check if we have a time field defined by the header keys TIMEZERO and DELTA_T.
+          //  If so then we create a new field called $TIME_FIELD, unless such a field already
+          //  exists, in which case we do nothing...
+          //
+          char charTimeZero[] = "TIMEZERO";
+
+          iStatus = 0;
+          iResult = fits_read_key( ffits, TDOUBLE, charTimeZero, &_dTimeZero, 0L, &iStatus );
+          if( iResult == 0 )
+          {
+            char charTimeDelta[] = "DELTA_T";
+
+            iResult = fits_read_key( ffits, TDOUBLE, charTimeDelta, &_dTimeDelta, 0L, &iStatus );
+            if( iResult == 0 )
+            {
+              if( _fieldList.find( QString( TIME_FIELD ) ) == _fieldList.end( ) )
+              {
+                _bHasTime = true;
+                _fieldList.append( TIME_FIELD );
+              }
+            }
+          }
+          
           if( lNumFrames * lMaxRepeat != _numFrames )
-	        {
+          {
             _numCols   = iNumCols;
             _numFrames = lNumFrames * lMaxRepeat;
             updateType = KstObject::UPDATE;
@@ -145,7 +181,7 @@ KstObject::UpdateType LFIIOSource::update( int u )
 
   updateNumFramesScalar();
 
-  return updateType;
+  return setLastUpdateResult(updateType);
 }
 
 
@@ -174,6 +210,14 @@ int LFIIOSource::readField( double *v, const QString& field, int s, int n )
     }
     iRead =  n;
   }
+  else if( _bHasTime && field == TIME_FIELD )
+  {
+    for( i = 0; i < n; i++ )
+    {
+      v[i] = _dTimeZero + ( (double)( s + i ) * _dTimeDelta );
+    }
+    iRead =  n;    
+  }
   else
   {
     memset( v, 0, n * sizeof( double ) );
@@ -223,7 +267,7 @@ bool LFIIOSource::getColNumber( const QString& field, int* piColNumber ) const
   iCol = field.toUInt( &bOk );
   if( bOk )
   {
-    if( iCol < _numCols )
+    if( iCol >= 0 && iCol < _numCols )
     {
       *piColNumber = iCol;
 
@@ -234,14 +278,20 @@ bool LFIIOSource::getColNumber( const QString& field, int* piColNumber ) const
   {
     iCount = _fieldList.count( );
 
-    for( i=0; i<iCount; i++ )
+    //
+    // start at 1, because the first entry is INDEX...
+    //
+    for( i=1; i<iCount; i++ )
     {
       strName = _fieldList[i].lower( );
       if( strName.compare( field.lower( ) ) == 0 )
       {
         bRetVal = true;
 
-        *piColNumber = i;
+        //
+        // subtract 1 to be consistent with the value returned from the previous branch...
+        //
+        *piColNumber = i-1;
 
         break;
       }
@@ -261,6 +311,10 @@ bool LFIIOSource::isValidField( const QString& field ) const
   {
     bRetVal = true;
   }
+  else if( field == TIME_FIELD && _bHasTime )
+  {
+    bRetVal = true;
+  }
   else
   {
     bRetVal = getColNumber( field, &iCol );
@@ -296,16 +350,16 @@ QString LFIIOSource::fileType( ) const
 }
 
 
-void LFIIOSource::save( QTextStream& ts )
+void LFIIOSource::save( QTextStream& ts, const QString& indent )
 {
-  KstDataSource::save( ts );
+  KstDataSource::save( ts, indent );
 }
 
 
 extern "C" {
-KstDataSource *create_lfiio( const QString& filename, const QString& type )
+KstDataSource *create_lfiio( KConfig *cfg, const QString& filename, const QString& type )
 {
-  return new LFIIOSource( filename, type );
+  return new LFIIOSource( cfg, filename, type );
 }
 
 QStringList provides_lfiio( )
@@ -317,18 +371,20 @@ QStringList provides_lfiio( )
   return rc;
 }
 
-int understands_lfiio( const QString& filename )
+int understands_lfiio( KConfig*, const QString& filename )
 {
   fitsfile* ffits;
   int       iStatus = 0;
-
+  int       iRetVal = 0;
+  
   //
   // determine if it is a FITS file...
   //
   if( fits_open_table( &ffits, filename.ascii( ), READONLY, &iStatus ) == 0 )
   {
     fits_close_file( ffits, &iStatus );
-    return 99;
+
+    iRetVal = 99;
   }
   else
   {
@@ -337,9 +393,11 @@ int understands_lfiio( const QString& filename )
     //
   }
 
-  return 0;
+  return iRetVal;
 }
 
 }
 
+KST_KEY_DATASOURCE_PLUGIN(lfiio)
+
 // vim: ts=2 sw=2 et
