@@ -262,7 +262,7 @@ bool PlanckSource::supportsTimeConversions() const {
 }
 
 
-int PlanckSource::sampleForTime(const QDateTime& time, bool *ok) {
+int PlanckSource::sampleForTime(const KST::ExtDateTime& time, bool *ok) {
   if (!_valid) {
     if (ok) {
       *ok = false;
@@ -270,12 +270,12 @@ int PlanckSource::sampleForTime(const QDateTime& time, bool *ok) {
     return -1;
   }
 
-  int tai = time.toTime_t() + 86400 * (365 * 12 + 3) + 32;
+  int tai = time.toTime_t() + 86400 * (365 * 12 + 3) + 32 + 3600;
   return _planckObject->sampleForTime(tai, ok);
 }
 
 
-int PlanckSource::sampleForTime(int seconds, bool *ok) {
+int PlanckSource::sampleForTime(double ms, bool *ok) {
   if (!_valid) {
     if (ok) {
       *ok = false;
@@ -283,16 +283,16 @@ int PlanckSource::sampleForTime(int seconds, bool *ok) {
     return -1;
   }
 
-  return _planckObject->sampleForTimeRelative(seconds, ok);
+  return _planckObject->sampleForTimeRelative(ms, ok);
 }
 
 
-QDateTime PlanckSource::timeForSample(int sample, bool *ok) {
+KST::ExtDateTime PlanckSource::timeForSample(int sample, bool *ok) {
   if (!_valid) {
     if (ok) {
       *ok = false;
     }
-    return QDateTime();
+    return KST::ExtDateTime();
   }
 
   double t = _planckObject->timeForSample(sample);
@@ -300,16 +300,16 @@ QDateTime PlanckSource::timeForSample(int sample, bool *ok) {
     if (ok) {
       *ok = false;
     }
-    return QDateTime();
+    return KST::ExtDateTime();
   }
 
-  QDateTime dt;
+  KST::ExtDateTime dt;
   dt.setTime_t(int(floor(t + 0.5)) - (86400 * (365 * 12 + 3) + 32));
   return dt;
 }
 
 
-int PlanckSource::relativeTimeForSample(int sample, bool *ok) {
+double PlanckSource::relativeTimeForSample(int sample, bool *ok) {
   if (!_valid) {
     if (ok) {
       *ok = false;
@@ -326,7 +326,7 @@ int PlanckSource::relativeTimeForSample(int sample, bool *ok) {
     return -1;
   }
 
-  return int(floor(t - i + 0.5));
+  return floor(t - i + 0.5) * 1000.0;
 }
 
 
@@ -382,4 +382,6 @@ QStringList fieldList_planck(KConfig*, const QString& filename, const QString& t
 
 }
 
+KST_KEY_DATASOURCE_PLUGIN(planck)
+
 // vim: ts=2 sw=2 et
