@@ -25,27 +25,34 @@
 #include <kdebug.h>
 #include <kjsembed/jsbinding.h>
 
-using namespace KJSEmbed;
-
 KstBindCurve::KstBindCurve(KJS::ExecState *exec, KstVCurvePtr d)
-: KstBinding("Curve"), _d(d) {
+: KstBindDataObject(exec, d.data(), "Curve") {
   KJS::Object o(this);
   addBindings(exec, o);
 }
 
 
 KstBindCurve::KstBindCurve(KJS::ExecState *exec, KJS::Object *globalObject)
-: KstBinding("Curve") {
+: KstBindDataObject(exec, globalObject, "Curve") {
   KJS::Object o(this);
   addBindings(exec, o);
   if (globalObject) {
-    globalObject->put(exec, "Curve", o);
+    KstBindDataObject::addFactory("Curve", KstBindCurve::bindFactory);
   }
 }
 
 
+KstBindDataObject *KstBindCurve::bindFactory(KJS::ExecState *exec, KstDataObjectPtr obj) {
+  KstVCurvePtr v = kst_cast<KstVCurve>(obj);
+  if (v) {
+    return new KstBindCurve(exec, v);
+  }
+  return 0L;
+}
+
+
 KstBindCurve::KstBindCurve(int id)
-: KstBinding("Curve Method", id) {
+: KstBindDataObject(id, "Curve Method") {
 }
 
 
@@ -58,6 +65,8 @@ KJS::Object KstBindCurve::construct(KJS::ExecState *exec, const KJS::List& args)
   if (args.size() > 0) {
     x = extractVector(exec, args[0]);
     if (!x) {
+      KJS::Object eobj = KJS::Error::create(exec, KJS::TypeError);
+      exec->setException(eobj);
       return KJS::Object();
     }
   }
@@ -65,6 +74,8 @@ KJS::Object KstBindCurve::construct(KJS::ExecState *exec, const KJS::List& args)
   if (args.size() > 1) {
     y = extractVector(exec, args[1]);
     if (!y) {
+      KJS::Object eobj = KJS::Error::create(exec, KJS::TypeError);
+      exec->setException(eobj);
       return KJS::Object();
     }
   }
@@ -72,6 +83,8 @@ KJS::Object KstBindCurve::construct(KJS::ExecState *exec, const KJS::List& args)
   if (args.size() > 2) {
     ex = extractVector(exec, args[2]);
     if (!ex) {
+      KJS::Object eobj = KJS::Error::create(exec, KJS::TypeError);
+      exec->setException(eobj);
       return KJS::Object();
     }
   }
@@ -79,6 +92,8 @@ KJS::Object KstBindCurve::construct(KJS::ExecState *exec, const KJS::List& args)
   if (args.size() > 3) {
     ey = extractVector(exec, args[3]);
     if (!ey) {
+      KJS::Object eobj = KJS::Error::create(exec, KJS::TypeError);
+      exec->setException(eobj);
       return KJS::Object();
     }
   }
@@ -86,6 +101,8 @@ KJS::Object KstBindCurve::construct(KJS::ExecState *exec, const KJS::List& args)
   if (args.size() > 4) {
     exm = extractVector(exec, args[4]);
     if (!exm) {
+      KJS::Object eobj = KJS::Error::create(exec, KJS::TypeError);
+      exec->setException(eobj);
       return KJS::Object();
     }
   }
@@ -93,6 +110,8 @@ KJS::Object KstBindCurve::construct(KJS::ExecState *exec, const KJS::List& args)
   if (args.size() > 5) {
     eym = extractVector(exec, args[5]);
     if (!eym) {
+      KJS::Object eobj = KJS::Error::create(exec, KJS::TypeError);
+      exec->setException(eobj);
       return KJS::Object();
     }
   }
@@ -103,6 +122,12 @@ KJS::Object KstBindCurve::construct(KJS::ExecState *exec, const KJS::List& args)
     return KJS::Object();
   }
 
+  if (!x || !y) { // force at least X and Y vectors
+    KJS::Object eobj = KJS::Error::create(exec, KJS::SyntaxError);
+    exec->setException(eobj);
+    return KJS::Object();
+  }
+
   QColor color = KstColorSequence::next();
   KstVCurvePtr d = new KstVCurve(QString::null, x, y, ex, ey, exm, eym, color);
 
@@ -138,7 +163,6 @@ static CurveBindings curveBindings[] = {
 
 
 static CurveProperties curveProperties[] = {
-  { "tagName", &KstBindCurve::setTagName, &KstBindCurve::tagName },
   { "color", &KstBindCurve::setColor, &KstBindCurve::color },
   { "xVector", &KstBindCurve::setXVector, &KstBindCurve::xVector },
   { "yVector", &KstBindCurve::setYVector, &KstBindCurve::yVector },
@@ -163,7 +187,7 @@ static CurveProperties curveProperties[] = {
 
 
 KJS::ReferenceList KstBindCurve::propList(KJS::ExecState *exec, bool recursive) {
-  KJS::ReferenceList rc = KstBinding::propList(exec, recursive);
+  KJS::ReferenceList rc = KstBindDataObject::propList(exec, recursive);
 
   for (int i = 0; curveProperties[i].name; ++i) {
     rc.append(KJS::Reference(this, KJS::Identifier(curveProperties[i].name)));
@@ -181,13 +205,13 @@ bool KstBindCurve::hasProperty(KJS::ExecState *exec, const KJS::Identifier& prop
     }
   }
 
-  return KstBinding::hasProperty(exec, propertyName);
+  return KstBindDataObject::hasProperty(exec, propertyName);
 }
 
 
 void KstBindCurve::put(KJS::ExecState *exec, const KJS::Identifier& propertyName, const KJS::Value& value, int attr) {
   if (!_d) {
-    KstBinding::put(exec, propertyName, value, attr);
+    KstBindDataObject::put(exec, propertyName, value, attr);
     return;
   }
 
@@ -202,13 +226,13 @@ void KstBindCurve::put(KJS::ExecState *exec, const KJS::Identifier& propertyName
     }
   }
 
-  KstBinding::put(exec, propertyName, value, attr);
+  KstBindDataObject::put(exec, propertyName, value, attr);
 }
 
 
 KJS::Value KstBindCurve::get(KJS::ExecState *exec, const KJS::Identifier& propertyName) const {
   if (!_d) {
-    return KstBinding::get(exec, propertyName);
+    return KstBindDataObject::get(exec, propertyName);
   }
 
   QString prop = propertyName.qstring();
@@ -221,7 +245,7 @@ KJS::Value KstBindCurve::get(KJS::ExecState *exec, const KJS::Identifier& proper
     }
   }
   
-  return KstBinding::get(exec, propertyName);
+  return KstBindDataObject::get(exec, propertyName);
 }
 
 
@@ -233,27 +257,36 @@ KJS::Value KstBindCurve::call(KJS::ExecState *exec, KJS::Object& self, const KJS
     return KJS::Undefined();
   }
 
-  KstBindCurve *imp = dynamic_cast<KstBindCurve*>(self.imp());
-  if (!imp) {
-    KJS::Object eobj = KJS::Error::create(exec, KJS::GeneralError);
-    exec->setException(eobj);
-    return KJS::Undefined();
+  int start = KstBindDataObject::methodCount();
+  if (id > start) {
+    KstBindCurve *imp = dynamic_cast<KstBindCurve*>(self.imp());
+    if (!imp) {
+      KJS::Object eobj = KJS::Error::create(exec, KJS::GeneralError);
+      exec->setException(eobj);
+      return KJS::Undefined();
+    }
+
+    return (imp->*curveBindings[id - start - 1].method)(exec, args);
   }
 
-  return (imp->*curveBindings[id - 1].method)(exec, args);
+  return KstBindDataObject::call(exec, self, args);
 }
 
 
 void KstBindCurve::addBindings(KJS::ExecState *exec, KJS::Object& obj) {
+  int start = KstBindDataObject::methodCount();
   for (int i = 0; curveBindings[i].name != 0L; ++i) {
-    KJS::Object o = KJS::Object(new KstBindCurve(i + 1));
+    KJS::Object o = KJS::Object(new KstBindCurve(i + start + 1));
     obj.put(exec, curveBindings[i].name, o, KJS::Function);
   }
 }
 
 
+#define makeCurve(X) dynamic_cast<KstVCurve*>(const_cast<KstObject*>(X.data()))
+
 KJS::Value KstBindCurve::point(KJS::ExecState *exec, const KJS::List& args) {
-  if (!_d) {
+  KstVCurvePtr d = makeCurve(_d);
+  if (!d) {
     KJS::Object eobj = KJS::Error::create(exec, KJS::GeneralError);
     exec->setException(eobj);
     return KJS::Undefined();
@@ -272,15 +305,16 @@ KJS::Value KstBindCurve::point(KJS::ExecState *exec, const KJS::List& args) {
     return KJS::Undefined();
   }
 
-  KstReadLocker rl(_d);
+  KstReadLocker rl(d);
   double x, y;
-  _d->point(i, x, y);
+  d->point(i, x, y);
   return KJS::Object(new KstBindPoint(exec, x, y));
 }
 
 
 KJS::Value KstBindCurve::xErrorPoint(KJS::ExecState *exec, const KJS::List& args) {
-  if (!_d) {
+  KstVCurvePtr d = makeCurve(_d);
+  if (!d) {
     KJS::Object eobj = KJS::Error::create(exec, KJS::GeneralError);
     exec->setException(eobj);
     return KJS::Undefined();
@@ -299,15 +333,16 @@ KJS::Value KstBindCurve::xErrorPoint(KJS::ExecState *exec, const KJS::List& args
     return KJS::Undefined();
   }
 
-  KstReadLocker rl(_d);
+  KstReadLocker rl(d);
   double x, y, e;
-  _d->getEXPoint(i, x, y, e);
+  d->getEXPoint(i, x, y, e);
   return KJS::Number(e);
 }
 
 
 KJS::Value KstBindCurve::yErrorPoint(KJS::ExecState *exec, const KJS::List& args) {
-  if (!_d) {
+  KstVCurvePtr d = makeCurve(_d);
+  if (!d) {
     KJS::Object eobj = KJS::Error::create(exec, KJS::GeneralError);
     exec->setException(eobj);
     return KJS::Undefined();
@@ -326,15 +361,16 @@ KJS::Value KstBindCurve::yErrorPoint(KJS::ExecState *exec, const KJS::List& args
     return KJS::Undefined();
   }
 
-  KstReadLocker rl(_d);
+  KstReadLocker rl(d);
   double x, y, e;
-  _d->getEYPoint(i, x, y, e);
+  d->getEYPoint(i, x, y, e);
   return KJS::Number(e);
 }
 
 
 KJS::Value KstBindCurve::xMinusErrorPoint(KJS::ExecState *exec, const KJS::List& args) {
-  if (!_d) {
+  KstVCurvePtr d = makeCurve(_d);
+  if (!d) {
     KJS::Object eobj = KJS::Error::create(exec, KJS::GeneralError);
     exec->setException(eobj);
     return KJS::Undefined();
@@ -353,15 +389,16 @@ KJS::Value KstBindCurve::xMinusErrorPoint(KJS::ExecState *exec, const KJS::List&
     return KJS::Undefined();
   }
 
-  KstReadLocker rl(_d);
+  KstReadLocker rl(d);
   double x, y, e;
-  _d->getEXMinusPoint(i, x, y, e);
+  d->getEXMinusPoint(i, x, y, e);
   return KJS::Number(e);
 }
 
 
 KJS::Value KstBindCurve::yMinusErrorPoint(KJS::ExecState *exec, const KJS::List& args) {
-  if (!_d) {
+  KstVCurvePtr d = makeCurve(_d);
+  if (!d) {
     KJS::Object eobj = KJS::Error::create(exec, KJS::GeneralError);
     exec->setException(eobj);
     return KJS::Undefined();
@@ -380,31 +417,13 @@ KJS::Value KstBindCurve::yMinusErrorPoint(KJS::ExecState *exec, const KJS::List&
     return KJS::Undefined();
   }
 
-  KstReadLocker rl(_d);
+  KstReadLocker rl(d);
   double x, y, e;
-  _d->getEYMinusPoint(i, x, y, e);
+  d->getEYMinusPoint(i, x, y, e);
   return KJS::Number(e);
 }
 
 
-void KstBindCurve::setTagName(KJS::ExecState *exec, const KJS::Value& value) {
-  if (value.type() != KJS::StringType) {
-    KJS::Object eobj = KJS::Error::create(exec, KJS::TypeError);
-    exec->setException(eobj);
-    return;
-  }
-  KstWriteLocker wl(_d);
-  _d->setTagName(value.toString(exec).qstring());
-}
-
-
-KJS::Value KstBindCurve::tagName(KJS::ExecState *exec) const {
-  Q_UNUSED(exec)
-  KstReadLocker rl(_d);
-  return KJS::String(_d->tagName());
-}
-
-
 void KstBindCurve::setColor(KJS::ExecState *exec, const KJS::Value& value) {
   QVariant cv = KJSEmbed::convertToVariant(exec, value);
   if (!cv.canCast(QVariant::Color)) {
@@ -412,141 +431,183 @@ void KstBindCurve::setColor(KJS::ExecState *exec, const KJS::Value& value) {
     exec->setException(eobj);
     return;
   }
-  KstWriteLocker rl(_d);
-  _d->setColor(cv.toColor());
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstWriteLocker rl(d);
+    d->setColor(cv.toColor());
+  }
 }
 
 
 KJS::Value KstBindCurve::color(KJS::ExecState *exec) const {
-  KstReadLocker rl(_d);
-  return KJSEmbed::convertToValue(exec, _d->color());
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    return KJSEmbed::convertToValue(exec, d->color());
+  }
+
+  return KJSEmbed::convertToValue(exec, QColor());
 }
 
 
 void KstBindCurve::setXVector(KJS::ExecState *exec, const KJS::Value& value) {
   KstVectorPtr vp = extractVector(exec, value);
   if (vp) {
-    KstWriteLocker wl(_d);
-    _d->setXVector(vp);
+    KstVCurvePtr d = makeCurve(_d);
+    if (d) {
+      KstWriteLocker wl(d);
+      d->setXVector(vp);
+    }
   }
 }
 
 
 KJS::Value KstBindCurve::xVector(KJS::ExecState *exec) const {
-  KstReadLocker rl(_d);
-  KstVectorPtr vp = _d->xVector();
-  if (!vp) {
-    return KJS::Undefined();
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    KstVectorPtr vp = d->xVector();
+    if (vp) {
+      return KJS::Object(new KstBindVector(exec, vp));
+    }
   }
-
-  return KJS::Object(new KstBindVector(exec, vp));
+  return KJS::Null();
 }
 
 
 void KstBindCurve::setYVector(KJS::ExecState *exec, const KJS::Value& value) {
   KstVectorPtr vp = extractVector(exec, value);
   if (vp) {
-    KstWriteLocker wl(_d);
-    _d->setYVector(vp);
+    KstVCurvePtr d = makeCurve(_d);
+    if (d) {
+      KstWriteLocker wl(d);
+      d->setYVector(vp);
+    }
   }
 }
 
 
 KJS::Value KstBindCurve::yVector(KJS::ExecState *exec) const {
-  KstReadLocker rl(_d);
-  KstVectorPtr vp = _d->yVector();
-  if (!vp) {
-    return KJS::Undefined();
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    KstVectorPtr vp = d->yVector();
+    if (!vp) {
+      return KJS::Object(new KstBindVector(exec, vp));
+    }
   }
-
-  return KJS::Object(new KstBindVector(exec, vp));
+  return KJS::Null();
 }
 
 
 void KstBindCurve::setXErrorVector(KJS::ExecState *exec, const KJS::Value& value) {
   KstVectorPtr vp = extractVector(exec, value);
   if (vp) {
-    KstWriteLocker wl(_d);
-    _d->setXError(vp);
+    KstVCurvePtr d = makeCurve(_d);
+    if (d) {
+      KstWriteLocker wl(d);
+      d->setXError(vp);
+    }
   }
 }
 
 
 KJS::Value KstBindCurve::xErrorVector(KJS::ExecState *exec) const {
-  KstReadLocker rl(_d);
-  KstVectorPtr vp = _d->xErrorVector();
-  if (!vp) {
-    return KJS::Undefined();
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    KstVectorPtr vp = d->xErrorVector();
+    if (vp) {
+      return KJS::Object(new KstBindVector(exec, vp));
+    }
   }
-
-  return KJS::Object(new KstBindVector(exec, vp));
+  return KJS::Null();
 }
 
 
 void KstBindCurve::setYErrorVector(KJS::ExecState *exec, const KJS::Value& value) {
   KstVectorPtr vp = extractVector(exec, value);
   if (vp) {
-    KstWriteLocker wl(_d);
-    _d->setYError(vp);
+    KstVCurvePtr d = makeCurve(_d);
+    if (d) {
+      KstWriteLocker wl(d);
+      d->setYError(vp);
+    }
   }
 }
 
 
 KJS::Value KstBindCurve::yErrorVector(KJS::ExecState *exec) const {
-  KstReadLocker rl(_d);
-  KstVectorPtr vp = _d->yErrorVector();
-  if (!vp) {
-    return KJS::Undefined();
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    KstVectorPtr vp = d->yErrorVector();
+    if (vp) {
+      return KJS::Object(new KstBindVector(exec, vp));
+    }
   }
-
-  return KJS::Object(new KstBindVector(exec, vp));
+  return KJS::Null();
 }
 
 
 void KstBindCurve::setXMinusErrorVector(KJS::ExecState *exec, const KJS::Value& value) {
-  KstVectorPtr vp = extractVector(exec, value);
-  if (vp) {
-    KstWriteLocker wl(_d);
-    _d->setXMinusError(vp);
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstVectorPtr vp = extractVector(exec, value);
+    if (vp) {
+      KstWriteLocker wl(d);
+      d->setXMinusError(vp);
+    }
   }
 }
 
 
 KJS::Value KstBindCurve::xMinusErrorVector(KJS::ExecState *exec) const {
-  KstReadLocker rl(_d);
-  KstVectorPtr vp = _d->xMinusErrorVector();
-  if (!vp) {
-    return KJS::Undefined();
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    KstVectorPtr vp = d->xMinusErrorVector();
+    if (vp) {
+      return KJS::Object(new KstBindVector(exec, vp));
+    }
   }
-
-  return KJS::Object(new KstBindVector(exec, vp));
+  return KJS::Null();
 }
 
 
 void KstBindCurve::setYMinusErrorVector(KJS::ExecState *exec, const KJS::Value& value) {
-  KstVectorPtr vp = extractVector(exec, value);
-  if (vp) {
-    KstWriteLocker wl(_d);
-    _d->setYMinusError(vp);
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstVectorPtr vp = extractVector(exec, value);
+    if (vp) {
+      KstWriteLocker wl(d);
+      d->setYMinusError(vp);
+    }
   }
 }
 
 
 KJS::Value KstBindCurve::yMinusErrorVector(KJS::ExecState *exec) const {
-  KstReadLocker rl(_d);
-  KstVectorPtr vp = _d->yMinusErrorVector();
-  if (!vp) {
-    return KJS::Undefined();
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    KstVectorPtr vp = d->yMinusErrorVector();
+    if (vp) {
+      return KJS::Object(new KstBindVector(exec, vp));
+    }
   }
-
-  return KJS::Object(new KstBindVector(exec, vp));
+  return KJS::Null();
 }
 
 
 KJS::Value KstBindCurve::samplesPerFrame(KJS::ExecState *exec) const {
   Q_UNUSED(exec)
-  KstReadLocker rl(_d);
-  return KJS::Number(_d->samplesPerFrame());
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    return KJS::Number(d->samplesPerFrame());
+  }
+  return KJS::Number(0);
 }
 
 
@@ -556,15 +617,22 @@ void KstBindCurve::setIgnoreAutoScale(KJS::ExecState *exec, const KJS::Value& va
     exec->setException(eobj);
     return;
   }
-  KstWriteLocker wl(_d);
-  _d->setIgnoreAutoScale(value.toBoolean(exec));
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstWriteLocker wl(d);
+    d->setIgnoreAutoScale(value.toBoolean(exec));
+  }
 }
 
 
 KJS::Value KstBindCurve::ignoreAutoScale(KJS::ExecState *exec) const {
   Q_UNUSED(exec)
-  KstReadLocker rl(_d);
-  return KJS::Boolean(_d->ignoreAutoScale());
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    return KJS::Boolean(d->ignoreAutoScale());
+  }
+  return KJS::Boolean(false);
 }
 
 
@@ -574,15 +642,22 @@ void KstBindCurve::setHasPoints(KJS::ExecState *exec, const KJS::Value& value) {
     exec->setException(eobj);
     return;
   }
-  KstWriteLocker wl(_d);
-  _d->setHasPoints(value.toBoolean(exec));
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstWriteLocker wl(d);
+    d->setHasPoints(value.toBoolean(exec));
+  }
 }
 
 
 KJS::Value KstBindCurve::hasPoints(KJS::ExecState *exec) const {
   Q_UNUSED(exec)
-  KstReadLocker rl(_d);
-  return KJS::Boolean(_d->hasPoints());
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    return KJS::Boolean(d->hasPoints());
+  }
+  return KJS::Boolean(false);
 }
 
 
@@ -592,15 +667,22 @@ void KstBindCurve::setHasLines(KJS::ExecState *exec, const KJS::Value& value) {
     exec->setException(eobj);
     return;
   }
-  KstWriteLocker wl(_d);
-  _d->setHasLines(value.toBoolean(exec));
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstWriteLocker wl(d);
+    d->setHasLines(value.toBoolean(exec));
+  }
 }
 
 
 KJS::Value KstBindCurve::hasLines(KJS::ExecState *exec) const {
   Q_UNUSED(exec)
-  KstReadLocker rl(_d);
-  return KJS::Boolean(_d->hasLines());
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    return KJS::Boolean(d->hasLines());
+  }
+  return KJS::Boolean(false);
 }
 
 
@@ -610,15 +692,22 @@ void KstBindCurve::setHasBars(KJS::ExecState *exec, const KJS::Value& value) {
     exec->setException(eobj);
     return;
   }
-  KstWriteLocker wl(_d);
-  _d->setHasBars(value.toBoolean(exec));
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstWriteLocker wl(d);
+    d->setHasBars(value.toBoolean(exec));
+  }
 }
 
 
 KJS::Value KstBindCurve::hasBars(KJS::ExecState *exec) const {
   Q_UNUSED(exec)
-  KstReadLocker rl(_d);
-  return KJS::Boolean(_d->hasBars());
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    return KJS::Boolean(d->hasBars());
+  }
+  return KJS::Boolean(false);
 }
 
 
@@ -629,15 +718,22 @@ void KstBindCurve::setLineWidth(KJS::ExecState *exec, const KJS::Value& value) {
     exec->setException(eobj);
     return;
   }
-  KstWriteLocker wl(_d);
-  _d->setLineWidth(i);
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstWriteLocker wl(d);
+    d->setLineWidth(i);
+  }
 }
 
 
 KJS::Value KstBindCurve::lineWidth(KJS::ExecState *exec) const {
   Q_UNUSED(exec)
-  KstReadLocker rl(_d);
-  return KJS::Number(_d->lineWidth());
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    return KJS::Number(d->lineWidth());
+  }
+  return KJS::Number(0);
 }
 
 
@@ -648,15 +744,22 @@ void KstBindCurve::setPointDensity(KJS::ExecState *exec, const KJS::Value& value
     exec->setException(eobj);
     return;
   }
-  KstWriteLocker wl(_d);
-  _d->setPointDensity(i);
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstWriteLocker wl(d);
+    d->setPointDensity(i);
+  }
 }
 
 
 KJS::Value KstBindCurve::pointDensity(KJS::ExecState *exec) const {
   Q_UNUSED(exec)
-  KstReadLocker rl(_d);
-  return KJS::Number(_d->pointDensity());
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    return KJS::Number(d->pointDensity());
+  }
+  return KJS::Number(0);
 }
 
 
@@ -667,15 +770,22 @@ void KstBindCurve::setLineStyle(KJS::ExecState *exec, const KJS::Value& value) {
     exec->setException(eobj);
     return;
   }
-  KstWriteLocker wl(_d);
-  _d->setLineStyle(i);
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstWriteLocker wl(d);
+    d->setLineStyle(i);
+  }
 }
 
 
 KJS::Value KstBindCurve::lineStyle(KJS::ExecState *exec) const {
   Q_UNUSED(exec)
-  KstReadLocker rl(_d);
-  return KJS::Number(_d->lineStyle());
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    return KJS::Number(d->lineStyle());
+  }
+  return KJS::Number(0);
 }
 
 
@@ -686,37 +796,58 @@ void KstBindCurve::setBarStyle(KJS::ExecState *exec, const KJS::Value& value) {
     exec->setException(eobj);
     return;
   }
-  KstWriteLocker wl(_d);
-  _d->setBarStyle(i);
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstWriteLocker wl(d);
+    d->setBarStyle(i);
+  }
 }
 
 
 KJS::Value KstBindCurve::barStyle(KJS::ExecState *exec) const {
   Q_UNUSED(exec)
-  KstReadLocker rl(_d);
-  return KJS::Number(_d->barStyle());
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    return KJS::Number(d->barStyle());
+  }
+  return KJS::Number(0);
 }
 
 
 KJS::Value KstBindCurve::topLabel(KJS::ExecState *exec) const {
   Q_UNUSED(exec)
-  KstReadLocker rl(_d);
-  return KJS::String(_d->topLabel());
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    return KJS::String(d->topLabel());
+  }
+  return KJS::String();
 }
 
 
 KJS::Value KstBindCurve::xLabel(KJS::ExecState *exec) const {
   Q_UNUSED(exec)
-  KstReadLocker rl(_d);
-  return KJS::String(_d->xLabel());
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    return KJS::String(d->xLabel());
+  }
+  return KJS::String();
 }
 
 
 KJS::Value KstBindCurve::yLabel(KJS::ExecState *exec) const {
   Q_UNUSED(exec)
-  KstReadLocker rl(_d);
-  return KJS::String(_d->yLabel());
+  KstVCurvePtr d = makeCurve(_d);
+  if (d) {
+    KstReadLocker rl(d);
+    return KJS::String(d->yLabel());
+  }
+  return KJS::String();
 }
 
 
+#undef makeCurve
+
 // vim: ts=2 sw=2 et
