@@ -23,19 +23,25 @@
 #include "kstbackbuffer.h"
 #include "kstbasecurve.h"
 #include "kstcolorsequence.h"
+#include "kstplotbase.h"
 #include "kstplotdefines.h"
-#include "kstimage.h"
-#include "kstlabel.h"
 #include "kstviewwidget.h"
+#include "kstviewlegend.h"
+#include "kst_export.h"
 
 #include <qvaluestack.h>
 
-class KstLegend;
-typedef KstSharedPtr<KstLegend> KstLegendPtr;
+namespace Equation {
+  class Node;
+}
+
 class Kst2DPlot;
 typedef KstObjectList<KstSharedPtr<Kst2DPlot> > Kst2DPlotList;
+class KstViewLabel;
+typedef KstSharedPtr<KstViewLabel> KstViewLabelPtr;
+class KstPlotLabel;
 
-enum KstScaleModeType { AUTO = 0, AC = 1, FIXED = 2, AUTOUP = 3, NOSPIKE = 4, AUTOBORDER = 5 };
+enum KstScaleModeType { AUTO = 0, AC = 1, FIXED = 2, AUTOUP = 3, NOSPIKE = 4, AUTOBORDER = 5, EXPRESSION = 6 };
 
 struct KstPlotScale {
   double xmin;
@@ -45,6 +51,7 @@ struct KstPlotScale {
   KstScaleModeType xscalemode;
   KstScaleModeType yscalemode;
   bool xlog, ylog;
+  QString xMinExp, xMaxExp, yMinExp, yMaxExp;
 };
 
 struct KstMarker {
@@ -55,7 +62,7 @@ struct KstMarker {
 
 typedef QValueList<KstMarker> KstMarkerList;
 
-enum KstMouseModeType { INACTIVE, XY_ZOOMBOX, Y_ZOOMBOX, X_ZOOMBOX, LABEL_TOOL, LAYOUT_TOOL };
+enum KstMouseModeType { INACTIVE, XY_ZOOMBOX, Y_ZOOMBOX, X_ZOOMBOX, LAYOUT_TOOL };
 
 struct KstMouse {
   KstMouse();
@@ -63,6 +70,7 @@ struct KstMouse {
   int label, minMove;
   QPoint lastLocation, pressLocation;  // for zooming primarily
   QPoint tracker; // for tracking the mouse location
+  QPoint lastGuideline; // for tracking the last guideline location
   QRect plotGeometry;
   bool zooming() const;
   void zoomStart(KstMouseModeType t, const QPoint& location);
@@ -72,7 +80,7 @@ struct KstMouse {
 };
 
 
-class Kst2DPlot : public KstPlotBase {
+class KST_EXPORT Kst2DPlot : public KstPlotBase {
   Q_OBJECT
 public:
   Kst2DPlot(const QString& in_tag = "SomePlot",
@@ -88,21 +96,21 @@ public:
   static bool checkRange(double& min_in, double& max_in);
   static bool checkLRange(double& min_in, double& max_in, bool isLog);
   static void genAxisTickLabel(QString& label, double z, bool isLog);
-  static double timezoneHrs( );
-  
+
   virtual UpdateType update(int update_counter);
   virtual void save(QTextStream& ts, const QString& indent = QString::null);
-  virtual void saveTag(QTextStream& ts, const QString& indent = QString::null);
-
+  virtual void saveAttributes(QTextStream& ts, const QString& indent = QString::null);
+  
   virtual bool popupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObjectPtr topLevelParent);
   virtual bool layoutPopupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObjectPtr topLevelParent);
 
+  void drawGraphicSelectionAt(QPainter& p, const QPoint& pos);
   void drawDotAt(QPainter& p, double x, double y);
   void drawPlusAt(QPainter& p, double x, double y);
-  void internalAlignment(KstPaintType type, QPainter& p, QRect& plotRegion);
+  void internalAlignment(KstPainter& p, QRect& plotRegion);
   void addCurve(KstBaseCurvePtr curve);
-  void addLabel(KstLabelPtr label);
   void removeCurve(KstBaseCurvePtr curve);
+  void clearCurves();
   void setXScaleMode(KstScaleModeType scalemode);
   void setYScaleMode(KstScaleModeType scalemode);
 
@@ -110,6 +118,12 @@ public:
   void popLineWidth();
   void pushCurveColor(const QColor& c);
   void popCurveColor();
+  void pushCurveHasPoints(bool yes);
+  void popCurveHasPoints();
+  void pushCurveHasLines(bool yes);
+  void popCurveHasLines();
+  void pushCurvePointDensity(int pointDensity);
+  void popCurvePointDensity();
   void pushPlotColors();
   void popPlotColors();
 
@@ -120,6 +134,11 @@ public:
   /** Set the y scale */
   bool setYScale(double ymin, double ymax);
 
+  // set the X Expressions
+  bool setXExpressions(const QString& minExp, const QString& maxExp);
+  // set the Y Expressions
+  bool setYExpressions(const QString& minExp, const QString& maxExp);
+
   /** Set the log_scale */
   bool setLScale(double xmin, double ymin, double xmax, double ymax);
   /** Set the X log_scale */
@@ -134,20 +153,28 @@ public:
   void setLog(bool x_log, bool y_log);
   bool isXLog() const;
   bool isYLog() const;
-  void setXAxisInterpretation( bool isXAxisInterpreted, KstAxisInterpretation xAxisInterpretation, KstAxisDisplay xAxisDisplay, bool xAxisTimezoneLocal, double xAxisTimezoneHrs);
-  void getXAxisInterpretation( bool& isXAxisInterpreted, KstAxisInterpretation& xAxisInterpretation, KstAxisDisplay& xAxisDisplay, bool& xAxisTimezoneLocal, double& xAxisTimezoneHrs) const;
-  void setYAxisInterpretation( bool isYAxisInterpreted, KstAxisInterpretation yAxisInterpretation, KstAxisDisplay yAxisDisplay, bool yAxisTimezoneLocal, double yAxisTimezoneHrs);
-  void getYAxisInterpretation( bool& isYAxisInterpreted, KstAxisInterpretation& yAxisInterpretation, KstAxisDisplay& yAxisDisplay, bool& yAxisTimezoneLocal, double& yAxisTimezoneHrs ) const;
+  void setXAxisInterpretation( bool isXAxisInterpreted, KstAxisInterpretation xAxisInterpretation, KstAxisDisplay xAxisDisplay);
+  void getXAxisInterpretation( bool& isXAxisInterpreted, KstAxisInterpretation& xAxisInterpretation, KstAxisDisplay& xAxisDisplay) const;
+  void setYAxisInterpretation( bool isYAxisInterpreted, KstAxisInterpretation yAxisInterpretation, KstAxisDisplay yAxisDisplay);
+  void getYAxisInterpretation( bool& isYAxisInterpreted, KstAxisInterpretation& yAxisInterpretation, KstAxisDisplay& yAxisDisplay) const;
   void getScale(double& xmin, double& ymin, double& xmax, double& ymax) const;
+
+  // get expressions for the scale
+  void getXScaleExps(QString& minExp, QString& maxExp);
+  void getYScaleExps(QString& minExp, QString& maxExp);
+
   void getLScale(double& xmin, double& ymin, double& xmax, double& ymax) const;
 
   KstScaleModeType xScaleMode() const;
   KstScaleModeType yScaleMode() const;
 
-  void setXLabel(const QString& xlabel);
-  void setYLabel(const QString& ylabel);
-  void setTopLabel(const QString& toplabel);
-  void initFonts(const QFont& font, int font_size = 0);
+  KstPlotLabel *xLabel() const;
+  KstPlotLabel *yLabel() const;
+  KstPlotLabel *topLabel() const;
+  KstPlotLabel *xTickLabel() const;
+  KstPlotLabel *yTickLabel() const;
+  KstPlotLabel *fullTickLabel() const;
+
 
   QRect GetPlotRegion() const;
   QRect GetWinRegion() const;
@@ -160,28 +187,14 @@ public:
 
   virtual QString menuTitle() const;
 
-  /** Labels */
-  KstLabelPtr XLabel;
-  KstLabelPtr YLabel;
-  KstLabelPtr TopLabel;
-  KstLabelPtr XTickLabel;
-  KstLabelPtr YTickLabel;
-  KstLabelPtr XFullTickLabel;
-  KstLabelPtr YFullTickLabel;
-  KstLegendPtr Legend;
-
   double _width;
   double _height;
   double _pos_x;
   double _pos_y;
 
-  /** Arbitrary Labels */
-  KstLabelList _labelList;
-
   KstBaseCurveList Curves;
-  KstImageList _images;
 
-  void GenerateDefaultLabels();
+  void generateDefaultLabels(bool xl = true, bool yl = true, bool zl = true);
 
   /* kstview tells kstplot where to offset the plot to */
   void setPixRect(const QRect& RelPlotRegion, const QRect& RelWinRegion, const QRect& RelPlotAndAxisRegion);
@@ -198,14 +211,11 @@ public:
   virtual void mouseReleaseEvent(QWidget *view, QMouseEvent *e);
   virtual void keyPressEvent(QWidget *view, QKeyEvent *e);
   virtual void keyReleaseEvent(QWidget *view, QKeyEvent *e);
-  virtual void dragEnterEvent(QWidget *view, QDragEnterEvent *e);
-  virtual void dropEvent(QWidget *view, QDropEvent *e);
-  virtual void dragMoveEvent(QWidget *view, QDragMoveEvent *e);
   virtual void wheelEvent(QWidget *view, QWheelEvent *e);
   virtual void setHasFocus(bool hasFocus);
-  virtual void removeFocus(QPainter& p);
 
   void cancelZoom(QWidget *view);
+  void zoomSelfYLocalMax(bool unused);
   bool moveSelfHorizontal(bool left);
   bool moveSelfVertical(bool up);
   bool zoomSelfHorizontal(bool in);
@@ -221,6 +231,7 @@ public:
   void yZoomOut(KstViewWidget *);
   void xZoomMax(KstViewWidget *);
   void yZoomMax(KstViewWidget *);
+  void yZoomLocalMax(KstViewWidget *);
   void zoomMax(KstViewWidget *);
   void xLogSlot(KstViewWidget *);
   void yLogSlot(KstViewWidget *);
@@ -243,10 +254,10 @@ public:
   void moveSelfToCenter(double center);
 
   // curve related plot marker functions
-  void setCurveToMarkers(KstBaseCurvePtr curve, bool risingDetect, bool fallingDetect);
+  void setCurveToMarkers(KstVCurvePtr curve, bool risingDetect, bool fallingDetect);
   bool hasCurveToMarkers() const;
   void removeCurveToMarkers();
-  KstBaseCurvePtr curveToMarkers() const;
+  KstVCurvePtr curveToMarkers() const;
   bool curveToMarkersRisingDetect() const;
   bool curveToMarkersFallingDetect() const;
 
@@ -255,16 +266,11 @@ public:
   bool xOffsetMode() const { return _xOffsetMode; }
   bool yOffsetMode() const { return _yOffsetMode; }
 
-  // images
-  void addImage(KstImagePtr inImage, bool set_dirty = true);
-  void removeImage(KstImagePtr inImage, bool set_dirty = true);
-  void removeAllImages(bool set_dirty = true);
-  bool hasImage(KstImagePtr inImage) const;
-
   // plot grid lines
-  void setGridLines(bool xMajor, bool yMajor, bool xMinor, bool yMinor,
-                    const QColor& majorColor, const QColor& minorColor,
-                    bool majorGridColorDefault, bool minorGridColorDefault);
+  void setGridLinesColor(const QColor& majorColor, const QColor& minorColor,
+                         bool majorGridColorDefault, bool minorGridColorDefault);
+  void setXGridLines(bool xMajor, bool xMinor);
+  void setYGridLines(bool yMajor, bool yMinor);
   bool hasXMajorGrid() const { return _xMajorGrid; }
   bool hasYMajorGrid() const { return _yMajorGrid; }
   bool hasXMinorGrid() const { return _xMinorGrid; }
@@ -285,6 +291,62 @@ public:
   int xMajorTicks() const { return _xMajorTicks; }
   int yMajorTicks() const { return _yMajorTicks; }
 
+  // set and get tick mark display options
+  void setXTicksInPlot(bool yes);
+  void setXTicksOutPlot(bool yes);
+  void setYTicksInPlot(bool yes);
+  void setYTicksOutPlot(bool yes);
+  bool xTicksInPlot() const;
+  bool xTicksOutPlot() const;
+  bool yTicksInPlot() const;
+  bool yTicksOutPlot() const;
+
+  bool suppressTop() const;
+  bool suppressBottom() const;
+  bool suppressLeft() const;
+  bool suppressRight() const;
+  void setSuppressTop(bool yes);
+  void setSuppressBottom(bool yes);
+  void setSuppressLeft(bool yes);
+  void setSuppressRight(bool yes);
+
+  // change EXPRESSION mode to FIXED mode if possible
+  void optimizeXExps();
+  void optimizeYExps();
+
+  // transformed opposite axes
+  void setXTransformedExp(const QString& exp);
+  void setYTransformedExp(const QString& exp);
+  const QString& xTransformedExp() const;
+  const QString& yTransformedExp() const;
+
+  // axes reversal
+  void setXReversed(bool yes);
+  void setYReversed(bool yes);
+  bool xReversed() const;
+  bool yReversed() const;
+
+  // change the color mode for the plot
+  // If any of pointStylePriority, lineStylePriority, lineWidthPriority are == -1, then the
+  // corresponding curve property is not cycled or altered. The remaining priorities must
+  // be in the range [0, X-1] with no duplicates, where X = number of priorities != -1
+  void changeToMonochrome(int pointStylePriority, int lineStylePriority, int lineWidthPriority,
+                          int maxLineWidth, int pointDensity, bool forPrint);
+  // undo changes made by changeToMonochrome
+  // PRE: pointStylePriority, lineStylePriority, lineWidthPriority must have same values as when
+  //      passed to changeToMonochrome (otherwise behaviour is not as expected)
+  bool undoChangeToMonochrome(int pointStylePriority, int lineStylePriority, int lineWidthPriority);
+
+  //convenience routines for working with viewLegends
+  KstViewLegendPtr legend();
+
+  KstViewLegendPtr getOrCreateLegend();
+  
+  virtual QRect contentsRect() const;
+
+  double verticalSizeFactor();
+  double horizontalSizeFactor();
+  
 signals:
   void modified();
 
@@ -295,17 +357,21 @@ public slots:
   virtual void copyObject();
   virtual void copyObjectQuietly(KstViewObject& parent, const QString& name = QString::null) const;
   void draw(); // draw into back buffer
-  void draw(QPainter &p, KstPaintType type, double resolutionEnhancement = 1); // This actually does the drawing
-  virtual void paint(KstPaintType type, QPainter& p);
+  void draw(KstPainter& p, double resolutionEnhancement = 1); // This actually does the drawing
+  void paintSelf(KstPainter& p, const QRegion& bounds);
+  void updateSelf();
   void editCurve(int id);
   void editObject(int id);
   void editVector(int id);
-  void editImage(int id);
-  void matchAxis(int id);
+  void editMatrix(int id);
+  void matchAxes(int id);
+  void matchXAxis(int id);
   void fitCurve(int id);
   void filterCurve(int id);
   void removeCurve(int id);
-  void removeImage(int id);
+
+  // used in layout mode
+  bool showDialog(KstTopLevelViewPtr invoker, bool isNew = false);
 
 protected slots:
   void menuMoveUp();
@@ -318,6 +384,7 @@ protected slots:
   void menuYZoomOut();
   void menuXZoomMax();
   void menuYZoomMax();
+  void menuYZoomLocalMax();
   void menuZoomMax();
   void menuXLogSlot();
   void menuYLogSlot();
@@ -329,10 +396,12 @@ protected slots:
   void menuNextMarker();
   void menuPrevMarker();
 
-protected:
-  virtual KstViewObjectFactoryMethod factory() const;
+  void timezoneChanged(const QString& tz, int utcOffset);
 
 private:
+  // for backwards compatibility with old labels - convert an old saved label to a KstViewLabel
+  KstViewLabelPtr convertLabelToViewLabel(const QDomElement &e);
+
   void updateDirtyFromLabels();
 
   template<class T> void updateTiedPlots(void (Kst2DPlot::*method)(T), T arg);
@@ -343,50 +412,53 @@ private:
   void drawCursorPos(QWidget *view);
   void drawCursorPos(QPainter& p);
   void updateMousePos(const QPoint& pos);
-  bool getNearestDataPoint(const QPoint& pos, double& newxpos, double& newypos, QString& name, double& xmin, double &xmax, double& ymin, double& ymax);
-  void highlightNearestDataPoint(bool repaint, QWidget *view, const QPoint& pos);
+  void getCursorPos(const QPoint& pos, double &xpos, double &ypos, double &xmin, double &xmax, double &ymin, double& ymax);
+  bool getNearestDataPoint(const QPoint& pos, QString& name, double &newxpos, double &newypos, double xpos, double ypos, double xmin, double xmax);
+  void highlightNearestDataPoint(bool repaint, KstPainter *p, const QPoint& pos);
   void updateTieBox(QPainter&);
-  int labelNumber(QMouseEvent *e);
   bool legendUnder(QMouseEvent *e);
   KstMouseModeType globalZoomType() const;
   void setCursorForMode(QWidget *view, KstMouseModeType mode, const QPoint& pos);
-  void setCursorForMode(QWidget *view); // gets the mode from the mouse
   void zoomRectUpdate(QWidget *view, KstMouseModeType t, int x, int y);
   inline void commonConstructor(const QString& in_tag,
-                                KstScaleModeType yscale,
                                 KstScaleModeType xscale,
+                                KstScaleModeType yscale,
                                 double xmin,
                                 double ymin,
                                 double xmax,
                                 double ymax,
                                 bool x_log = false,
                                 bool y_log = false);
+  QPair<double, double> computeAutoBorder(bool log, double min, double max);
   void setBorders(double& xleft_bdr_px, double& xright_bdr_px,
                          double& ytop_bdr_px, double& ybot_bdr_px,
                          TickParameters &tpx,  TickParameters &tpy,
-                         QPainter& p, bool& bOffsetX, bool& bOffsetY);
+                         QPainter& p, bool& bOffsetX, bool& bOffsetY,
+                         double xtick_len_px, double ytick_len_px);
   void setTicks(double& tick, double& org,
                 double max, double min, bool is_log, bool isX, int base);
   double convertTimeValueToJD(KstAxisInterpretation axisInterpretation, double valueIn);
   double convertTimeDiffValueToDays(KstAxisInterpretation axisInterpretation, double diffIn);
-  void convertJDToDateString(KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, bool axisTimezoneLocal, double axisTimezoneHrs, QString& label, uint& length, double dJD);
-  void convertTimeValueToString(KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, bool axisTimezoneLocal, double axisTimezoneHrs, QString& label, uint& length, double z, bool isLog);
+  void convertJDToDateString(KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, QString& label, uint& length, double dJD);
+  void convertTimeValueToString(KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, QString& label, uint& length, double z, bool isLog);
   void convertDiffTimevalueToString(KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, double& zdiff, double zbase, double zvalue, bool isLog, double scale);
-  void genAxisTickLabelFullPrecision(KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, bool axisTimezoneLocal, double axisTimezoneHrs, QString& label, uint& length, double z, bool isLog, bool isInterpreted);
+  void genAxisTickLabelFullPrecision(KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, QString& label, uint& length, double z, bool isLog, bool isInterpreted);
   void genAxisTickLabelDifference(KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, QString& label, double zbase, double zvalue, bool isLog, bool isInterpreted, double scale);
   void getPrefixUnitsScale(bool isInterpreted, KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, bool bLog, double Min, double Max, double& range, double& scale, int& base, QString& strPrefix, QString& strUnits);
   void genOffsetLabel(KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, QString& label, double zbase, double zvalue, double Min, double Max, bool isLog, bool isInterpreted);
-  void genAxisTickLabels(QPainter& p, TickParameters &tp, double Min, double Max, bool bLog, KstLabelPtr Label,
-                         KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, 
-                         bool axisTimezoneLocal, double axisTimezoneHours, 
-                         bool isX, bool isInterpreted, bool isOffsetMode);
+  void genAxisTickLabels(TickParameters &tp, double Min, double Max, bool bLog, KstAxisInterpretation axisInterpretation, KstAxisDisplay axisDisplay, bool isX, bool isInterpreted, bool isOffsetMode);
+
+  // helper function needed before setBorders
+  void set2dPlotTickPix(double& xtickpix, double& ytickpix, int x_pix, int y_pix);
+
+  void updateXYGuideline(QWidget *view, const QPoint& oldPos, const QPoint& newPos, const QRect& pr, KstMouseModeType gzType);
 
   // range and domain of plot: not plot dimentions
   double XMin, XMax, YMin, YMax;
+
   double _copy_x, _copy_y;
   double _cursor_x, _cursor_y;
-  bool _cursorOffset;
-  
+
   // grid lines
   QColor _majorGridColor;
   QColor _minorGridColor;
@@ -397,6 +469,8 @@ private:
   bool _majorGridColorDefault : 1;
   bool _minorGridColorDefault : 1;
 
+  bool _cursorOffset : 1;
+
   bool _xLog : 1;
   bool _yLog : 1;
   bool _isTied : 1;
@@ -408,18 +482,39 @@ private:
   bool _xOffsetMode : 1;
   bool _yOffsetMode : 1;
 
-  int _draggableLabel : 15; // I think this should be enough
+  // tick mark display options
+  bool _xTicksInPlot : 1;
+  bool _xTicksOutPlot : 1;
+  bool _yTicksInPlot : 1;
+  bool _yTicksOutPlot : 1;
+
+  // suppress labels and axes
+  bool _suppressTop : 1;
+  bool _suppressBottom : 1;
+  bool _suppressLeft : 1;
+  bool _suppressRight : 1;
+
+  bool _xMinParsedValid : 1, _xMaxParsedValid : 1, _yMinParsedValid : 1, _yMaxParsedValid : 1;
+  bool _xTransformed : 1, _yTransformed : 1;
+
+  bool _isXAxisInterpreted : 1, _isYAxisInterpreted : 1;
+
+  // reverse axes
+  bool _xReversed : 1, _yReversed : 1;
+
+  bool _drawingGraphics : 1;
 
-  bool _isXAxisInterpreted, _isYAxisInterpreted;
   KstAxisInterpretation _xAxisInterpretation, _yAxisInterpretation;
   KstAxisDisplay _xAxisDisplay, _yAxisDisplay;
-  bool _xAxisTimezoneLocal, _yAxisTimezoneLocal;
-  double _xAxisTimezoneHrs, _yAxisTimezoneHrs;
 
   KstScaleModeType _xScaleMode, _yScaleMode;
 
   QPtrList<KstPlotScale> _plotScaleList;
 
+  // hold last minimum and maximum y values that were plotted
+  double _yPlottedMinCached, _yPlottedMaxCached;
+
+
   /** Stores border limits.  Set by KstPlot::paint().
       Stored here to be Used to determine mouse mode */
   QRect PlotRegion;
@@ -427,11 +522,10 @@ private:
   QRect PlotAndAxisRegion;
 
   void updateScale();
+  void adjustFontSize();
 
   KstMouse _mouse;
-  QPoint _draggablePoint;
-
-  QMap<int, QString> _curveEditMap, _curveFitMap, _curveRemoveMap, _imageEditMap, _imageRemoveMap, _objectEditMap;
+  QMap<int, QString> _curveEditMap, _curveFitMap, _curveRemoveMap, _objectEditMap;
   QMap<int, QGuardedPtr<Kst2DPlot> > _plotMap;
 
   KstBackBuffer _buffer;
@@ -445,8 +539,8 @@ private:
 
   // plot markers. This needs to remain sorted
   KstMarkerList _plotMarkers;
-  // the curve used for auto marker generation
-  KstBaseCurvePtr _curveToMarkers;
+  // the vcurve used for auto marker generation
+  KstVCurvePtr _curveToMarkers;
   void updateMarkersFromCurve();  //auto-generates the markers from _curveToMarkers
 
   // minor tick marks.
@@ -458,35 +552,47 @@ private:
   int _xMajorTicks;
   int _yMajorTicks;
 
+  KstPlotLabel *_xLabel, *_yLabel, *_topLabel, *_xTickLabel, *_yTickLabel, *_fullTickLabel;
+
   QValueStack<QColor> _colorStack;
 
+  // for range expressions
+  bool reparse(const QString& stringExp, Equation::Node** eqNode);
+  bool reparseToText(QString& stringExp);
+  Equation::Node* _xMinParsed;
+  Equation::Node* _xMaxParsed;
+  Equation::Node* _yMinParsed;
+  Equation::Node* _yMaxParsed;
+
+  // expressions for range and domain of plot
+  QString _xMinExp, _xMaxExp, _yMinExp, _yMaxExp;
+
+  // for the transformed axes
+  QString _xTransformedExp, _yTransformedExp;
+
   // helper functions for draw(...)
   void plotLabels(QPainter& p, int x_px, int y_px, double xleft_bdr_px, double ytop_bdr_px);
   void plotAxes(QPainter& p, QRect& plotRegion,
-      TickParameters tpx, 
+      TickParameters tpx,
       double xleft_bdr_px, double xright_bdr_px,
       double x_orig_px, double xtick_px,
       double xtick_len_px, int x_px,
-      TickParameters tpy, 
+      TickParameters tpy,
       double ytop_bdr_px, double ybot_bdr_px,
       double y_orig_px, double ytick_px,
       double ytick_len_px, int y_px,
-      bool offsetX, bool offsetY);
+      bool offsetY);
   void plotGridLines(QPainter& p,
       double XTick, double xleft_bdr_px, double xright_bdr_px,
       double x_orig_px, double xtick_px, double xtick_len_px, int x_px,
       double YTick, double ytop_bdr_px, double ybot_bdr_px,
       double y_orig_px, double ytick_px, double ytick_len_px, int y_px);
-  void plotCurves(QPainter& p,
-      double Lx, double Hx, double Ly, double Hy,
-      double m_X, double m_Y, double b_X, double b_Y, int penWidth);
-  void plotImages(QPainter& p,
-      double Lx, double Hx, double Ly, double Hy,
-      double m_X, double m_Y, double b_X, double b_Y,
-      double x_max, double y_max, double x_min, double y_min);
   void plotPlotMarkers(QPainter& p,
       double b_X, double b_Y, double x_max, double x_min,
-      double y_px, double ytop_bdr_px, double ybot_bdr_px);
+      double y_px, double ytop_bdr_px, double ybot_bdr_px, int penWidth);
+
+  KstScaleModeType _xScaleModeDefault;
+  KstScaleModeType _yScaleModeDefault;
 };
 
 typedef KstSharedPtr<Kst2DPlot> Kst2DPlotPtr;
