@@ -17,8 +17,12 @@
 
 #include "kstborderedviewobject.h"
 
+#include <kglobal.h>
+#include <klocale.h>
+
 #include <qpainter.h>
 #include <qstylesheet.h>
+#include <qvariant.h>
 
 KstBorderedViewObject::KstBorderedViewObject(const QString& type)
 : KstViewObject(type), _borderColor(QColor(0, 0, 0)), _borderWidth(0), _padding(0), _margin(0) {
@@ -28,9 +32,9 @@ KstBorderedViewObject::KstBorderedViewObject(const QString& type)
 KstBorderedViewObject::KstBorderedViewObject(const QDomElement& e)
 : KstViewObject(e), _borderColor(QColor(0, 0, 0)), _borderWidth(0), _padding(0), _margin(0) {
   QDomNode n = e.firstChild();
-  while( !n.isNull() ) {
+  while (!n.isNull()) {
     QDomElement el = n.toElement(); // try to convert the node to an element.
-    if( !el.isNull() ) { // the node was really an element.
+    if (!el.isNull()) { // the node was really an element.
       if (el.tagName() == "border") {
         _borderColor.setNamedColor(el.attribute( "color", "#7f0000" ));
         _borderWidth = el.attribute( "width", "0" ).toInt();
@@ -56,44 +60,59 @@ KstBorderedViewObject::~KstBorderedViewObject() {
 }
 
 
-void KstBorderedViewObject::save(QTextStream& ts, const QString& indent) {
+void KstBorderedViewObject::saveAttributesOnly(QTextStream& ts, const QString& indent) {
   ts << indent << "<border color=\""
     << QStyleSheet::escape(_borderColor.name())
     << "\" width=\"" << _borderWidth
     << "\" padding=\"" << _padding
     << "\" margin=\"" << _margin << "\" />" << endl;
+}
+
+
+void KstBorderedViewObject::save(QTextStream& ts, const QString& indent) {
+  saveAttributesOnly(ts, indent);
   KstViewObject::save(ts, indent);
 }
 
 
-void KstBorderedViewObject::saveTag(QTextStream& ts, const QString& indent) {
-  ts << indent << "<border color=\""
-    << QStyleSheet::escape(_borderColor.name())
-    << "\" width=\"" << _borderWidth
-    << "\" padding=\"" << _padding
-    << "\" margin=\"" << _margin << "\" />" << endl;
-  KstViewObject::saveTag(ts, indent);
+void KstBorderedViewObject::saveAttributes(QTextStream& ts, const QString& indent) {
+  saveAttributesOnly(ts, indent);
+  KstViewObject::saveAttributes(ts, indent);
 }
 
 
-void KstBorderedViewObject::paint(KstPaintType type, QPainter& p) {
-  KstViewObject::paint(type, p);
-  if (_borderWidth > 0 && !_focus && !_selected) {
+void KstBorderedViewObject::paintSelf(KstPainter& p, const QRegion& bounds) {
+  p.save();
+  if (p.type() != KstPainter::P_PRINT && p.type() != KstPainter::P_EXPORT) {
+    if (p.makingMask()) {
+      p.setRasterOp(Qt::SetROP);
+      KstViewObject::paintSelf(p, bounds);
+    } else {
+      const QRegion clip(clipRegion());
+      KstViewObject::paintSelf(p, bounds - clip);
+      p.setClipRegion(bounds & clip);
+    }
+  }
+  if (_borderWidth > 0) {
     QRect r;
     QPen pen(_borderColor, _borderWidth);
-
+    p.setBrush(Qt::NoBrush);
     p.setPen(pen);
-    r.setX(_geom.left() + _margin);
-    r.setY(_geom.top() + _margin);
-    r.setWidth(_geom.width() - 2 * _margin);
-    r.setHeight(_geom.height() - 2 * _margin);
+    r.setX(_geom.left() + _margin + _borderWidth / 2);
+    r.setY(_geom.top() + _margin + _borderWidth / 2);
+    r.setWidth(_geom.width() - 2 * _margin - _borderWidth + 1);
+    r.setHeight(_geom.height() - 2 * _margin - _borderWidth + 1);
     p.drawRect(r);
   }
+  p.restore();
 }
 
 
 void KstBorderedViewObject::setBorderColor(const QColor& c) {
-  _borderColor = c;
+  if (_borderColor != c) {
+    setDirty();
+    _borderColor = c;
+  }
 }
 
 
@@ -103,7 +122,11 @@ const QColor& KstBorderedViewObject::borderColor() const {
 
 
 void KstBorderedViewObject::setBorderWidth(int w) {
-  _borderWidth = w;
+  int mw = kMax(0, w);
+  if (_borderWidth != mw) {
+    _borderWidth = mw;
+    setDirty();
+  }
 }
 
 
@@ -113,7 +136,11 @@ int KstBorderedViewObject::borderWidth() const {
 
 
 void KstBorderedViewObject::setMargin(int w) {
-  _margin = w;
+  int mm = kMax(0, w);
+  if (_margin != mm) {
+    _margin = mm;
+    setDirty();
+  }
 }
 
 
@@ -123,7 +150,11 @@ int KstBorderedViewObject::margin() const {
 
 
 void KstBorderedViewObject::setPadding(int p) {
-  _padding = p;
+  int mp = kMax(0, p);
+  if (_padding != mp) {
+    _padding = mp;
+    setDirty();
+  }
 }
 
 
@@ -137,18 +168,18 @@ QRect KstBorderedViewObject::contentsRect() const {
   int mpb = _margin + _padding + _borderWidth;
   rc.setX(_geom.left() + mpb);
   rc.setY(_geom.top() + mpb);
-  rc.setWidth(_geom.width() + 2 * mpb);
-  rc.setHeight(_geom.height() + 2 * mpb);
+  rc.setWidth(_geom.width() - 2 * mpb);
+  rc.setHeight(_geom.height() - 2 * mpb);
   return rc;
 }
 
 
 void KstBorderedViewObject::setContentsRect(QRect& rect) {
-  int mpb = _margin + _padding + _borderWidth;
+  const int mpb = _margin + _padding + _borderWidth;
   _geom.setX(rect.left() - mpb);
   _geom.setY(rect.top() - mpb);
-  _geom.setWidth(rect.width() - 2 * mpb);
-  _geom.setHeight(rect.height() - 2 * mpb);
+  _geom.setWidth(rect.width() + 2 * mpb);
+  _geom.setHeight(rect.height() + 2 * mpb);
 }
 
 
@@ -164,5 +195,43 @@ void KstBorderedViewObject::readBinary(QDataStream& str) {
 }
 
 
+QMap<QString, QVariant> KstBorderedViewObject::widgetHints(const QString& propertyName) const {
+  QMap<QString, QVariant> map = KstViewObject::widgetHints(propertyName);
+  if (!map.empty()) {
+    return map;
+  }
+
+  if (propertyName == "borderColor") {
+    map.insert(QString("_kst_widgetType"), QString("KColorButton"));
+    map.insert(QString("_kst_label"), i18n("Border color"));
+  } else if (propertyName == "borderWidth") {
+    map.insert(QString("_kst_widgetType"), QString("QSpinBox"));
+    map.insert(QString("_kst_label"), i18n("Border width"));
+    map.insert(QString("minValue"), 0);
+  }
+  return map;
+}
+
+
+void KstBorderedViewObject::setBackgroundColor(const QColor& color) {
+  KstViewObject::setBackgroundColor(color);
+}
+
+
+QColor KstBorderedViewObject::backgroundColor() const {
+  return KstViewObject::backgroundColor();
+}
+
+
+void KstBorderedViewObject::setForegroundColor(const QColor& color) {
+  KstViewObject::setForegroundColor(color);
+}
+
+
+QColor KstBorderedViewObject::foregroundColor() const {
+  return KstViewObject::foregroundColor();
+}
+
+
 #include "kstborderedviewobject.moc"
 // vim: ts=2 sw=2 et
