@@ -19,7 +19,9 @@
 #include <qlistbox.h>
 #include <qpushbutton.h>
 #include <qspinbox.h>
+#include <qtimer.h>
 
+#include <kcombobox.h>
 #include <knuminput.h>
 
 #include "datarangewidget.h"
@@ -42,35 +44,45 @@ KstChangeNptsDialogI::KstChangeNptsDialogI(QWidget* parent,
             this,      SLOT(updateDefaults( int )));
 }
 
+
 KstChangeNptsDialogI::~KstChangeNptsDialogI() {
 }
 
+
 void KstChangeNptsDialogI::selectAll() {
   CurveList->selectAll(true);
 }
 
+
 bool KstChangeNptsDialogI::updateChangeNptsDialog() {
   QStringList qsl;
-
+  int inserted = 0;
+  
   for (uint i_vector = 0; i_vector < CurveList->count(); i_vector++) {
     if (CurveList->isSelected(i_vector)) {
       qsl.append(CurveList->text(i_vector));
     }
   }
   CurveList->clear();
+  
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
-  /* insert vectors into ChangeNptsCurveList */
-  for (uint i = 0; i < rvl.count(); i++) {
-    rvl[i]->readLock();
-    CurveList->insertItem(rvl[i]->tagName(), -1);
-    if (qsl.contains(rvl[i]->tagName())) {
-      CurveList->setSelected(i, true);
+  // insert vectors into ChangeNptsCurveList
+  CurveList->blockSignals(true); // avoid deadlock
+  for (KstRVectorList::ConstIterator i = rvl.begin(); i != rvl.end(); ++i) {
+    KstRVectorPtr vector = *i;
+    vector->readLock();
+    CurveList->insertItem(vector->tagName(), -1);
+    if (qsl.contains(vector->tagName())) {
+      CurveList->setSelected(inserted, true);
     }
-    rvl[i]->readUnlock();
+    ++inserted;
+    vector->readUnlock();
   }
+  CurveList->blockSignals(false);
   return !qsl.isEmpty();
 }
 
+
 void KstChangeNptsDialogI::showChangeNptsDialog() {
   bool some_slected = updateChangeNptsDialog();
   updateDefaults(0);
@@ -81,60 +93,86 @@ void KstChangeNptsDialogI::showChangeNptsDialog() {
   raise();
 }
 
+
 void KstChangeNptsDialogI::applyNptsChange() {
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
-  for (uint i_vector = 0; i_vector < CurveList->count(); i_vector++) {
-    if (CurveList->isSelected(i_vector)) {
-      KstRVectorPtr vector = rvl[i_vector];
-      vector->readLock();
-      int f0, n;
-      if (_kstDataRange->isTime() && vector->dataSource()) {
-        vector->dataSource()->readLock();
-        f0 = vector->dataSource()->sampleForTime(_kstDataRange->f0Value());
-        n = vector->dataSource()->sampleForTime(_kstDataRange->nValue());
-        vector->dataSource()->readUnlock();
-      } else {
-        f0 = _kstDataRange->f0Value();
-        n = _kstDataRange->nValue();
+  for (uint i = 0; i < CurveList->count(); ++i) {
+    if (CurveList->isSelected(i)) {
+      KstRVectorPtr vector = *(rvl.findTag(CurveList->text(i)));
+      if (vector) {
+        int f0, n;
+        
+        vector->readLock();
+        KstDataSourcePtr ds = vector->dataSource();
+        if (_kstDataRange->isStartRelativeTime() && ds) {
+          ds->readLock();
+          f0 = ds->sampleForTime(_kstDataRange->f0Value());
+          ds->readUnlock();
+        } else if (_kstDataRange->isStartAbsoluteTime() && ds) {
+          ds->readLock();
+          f0 = ds->sampleForTime(_kstDataRange->f0DateTimeValue());
+          ds->readUnlock();
+        } else {
+          f0 = int(_kstDataRange->f0Value());
+        }
+      
+        if (_kstDataRange->isRangeRelativeTime() && ds) {
+          ds->readLock();
+          double nValStored = _kstDataRange->nValue();
+          if (_kstDataRange->CountFromEnd->isChecked()) {
+            int frameCount = ds->frameCount(vector->field());
+            double msCount = ds->relativeTimeForSample(frameCount - 1);
+            n = frameCount - 1 - ds->sampleForTime(msCount - nValStored);
+          } else {
+            double fTime = ds->relativeTimeForSample(f0);
+            n = ds->sampleForTime(fTime + nValStored) - ds->sampleForTime(fTime);
+          }
+          ds->readUnlock();
+        } else {
+          n = int(_kstDataRange->nValue());
+        }
+        vector->readUnlock();
+        
+        vector->writeLock();
+        vector->changeFrames(
+          (_kstDataRange->CountFromEnd->isChecked() ? -1 : f0),
+          (_kstDataRange->ReadToEnd->isChecked() ? -1 : n),
+          _kstDataRange->Skip->value(),
+          _kstDataRange->DoSkip->isChecked(),
+          _kstDataRange->DoFilter->isChecked());
+        vector->writeUnlock();
       }
-      vector->readUnlock();
-      vector->writeLock();
-      vector->changeFrames(
-        (_kstDataRange->CountFromEnd->isChecked() ? -1 : f0),
-        (_kstDataRange->ReadToEnd->isChecked() ? -1 : n),
-        _kstDataRange->Skip->value(),
-        _kstDataRange->DoSkip->isChecked(),
-        _kstDataRange->DoFilter->isChecked());
-      vector->writeUnlock();
     }
   }
 
+  // avoid re-entering the dialog
+  QTimer::singleShot(0, this, SLOT(emitDocChanged()));
+}
+
+
+void KstChangeNptsDialogI::emitDocChanged() {
   emit docChanged();
 }
 
+
 void KstChangeNptsDialogI::updateDefaults(int index) {
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
-  if (rvl.isEmpty() || index >= (int)rvl.count()) {
+  if (rvl.isEmpty() || index >= (int)rvl.count() || index < 0) {
     return;
   }
 
   KstRVectorPtr vector = rvl[index];
-  vector->writeLock();
+  vector->readLock();
+
+  _kstDataRange->_startUnits->setCurrentItem(0);
+  _kstDataRange->_rangeUnits->setCurrentItem(0);
 
   /* fill the vector range entries */
-  if (vector->countFromEOF()) {
-    _kstDataRange->CountFromEnd->setChecked(true);
-  } else {
-    _kstDataRange->CountFromEnd->setChecked(false);
-  }
+  _kstDataRange->CountFromEnd->setChecked(vector->countFromEOF());
   _kstDataRange->setF0Value(vector->reqStartFrame());
 
   /* fill number of frames entries */
-  if (vector->readToEOF()) {
-    _kstDataRange->ReadToEnd->setChecked(true);
-  } else {
-    _kstDataRange->ReadToEnd->setChecked(false);
-  }
+  _kstDataRange->ReadToEnd->setChecked(vector->readToEOF());
   _kstDataRange->setNValue(vector->reqNumFrames());
 
   /* fill in frames to skip box */
@@ -142,7 +180,34 @@ void KstChangeNptsDialogI::updateDefaults(int index) {
   _kstDataRange->DoSkip->setChecked(vector->doSkip());
   _kstDataRange->DoFilter->setChecked(vector->doAve());
   _kstDataRange->updateEnables();
-  vector->writeUnlock();
+
+  vector->readUnlock();
+}
+
+
+void KstChangeNptsDialogI::updateTimeCombo() {
+  KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
+  uint cnt = CurveList->count();
+  bool supportsTime = true;
+  for (uint i = 0; i < cnt; ++i) {
+    if (CurveList->isSelected(i)) {
+      KstRVectorPtr vector = *(rvl.findTag(CurveList->text(i)));
+      if (vector) {
+        vector->readLock();
+        KstDataSourcePtr ds = vector->dataSource();
+        vector->readUnlock();
+        if (ds) {
+          ds->readLock();
+          supportsTime = ds->supportsTimeConversions();
+          ds->readUnlock();
+          if (!supportsTime) {
+            break;
+          }
+        }
+      }
+    }
+  }
+  _kstDataRange->setAllowTime(supportsTime);
 }
 
 
