@@ -22,17 +22,25 @@
 #include "kstcurvehint.h"
 #include "kststring.h"
 #include "kstvector.h"
+#include "kstmatrix.h"
+#include "kst_export.h"
 
-class KstDataObject : public KstObject {
+typedef KstSharedPtr<KstDataObject> KstDataObjectPtr;
+typedef KstObjectList<KstDataObjectPtr> KstDataObjectList;
+typedef QMap<KstDataObjectPtr, KstDataObjectPtr> KstDataObjectDataObjectMap;
+
+class KST_EXPORT KstDataObject : public KstObject {
   Q_OBJECT
   public:
     KstDataObject();
-    KstDataObject(QDomElement& e);
+    KstDataObject(const QDomElement& e);
+    KstDataObject(const KstDataObject& object);
     virtual ~KstDataObject();
 
     virtual UpdateType update(int updateCounter = -1) = 0;
     virtual const QString& typeString() const { return _typeString; }
     virtual QString propertyString() const = 0;
+    virtual const QString& type() const { return _type; }
 
     virtual int sampleCount() const { return 0; }
 
@@ -52,6 +60,11 @@ class KstDataObject : public KstObject {
     const KstStringMap& outputStrings() const { return _outputStrings; }
     KstStringMap& inputStrings() { return _inputStrings;  }
     KstStringMap& outputStrings() { return _outputStrings; }
+    
+    const KstMatrixMap& inputMatrices() const { return _inputMatrices; }
+    const KstMatrixMap& outputMatrices() const { return _outputMatrices; }
+    KstMatrixMap& inputMatrices() { return _inputMatrices; }
+    KstMatrixMap& outputMatrices() { return _outputMatrices; }
 
     virtual void save(QTextStream& ts, const QString& indent = QString::null);
 
@@ -69,8 +82,16 @@ class KstDataObject : public KstObject {
     virtual bool isValid() const;
 
     virtual const KstCurveHintList* curveHints() const;
-
+    
     virtual bool deleteDependents();
+    
+    bool duplicateDependents(KstDataObjectDataObjectMap& duplicatedMap);
+    
+    virtual KstDataObjectPtr makeDuplicate(KstDataObjectDataObjectMap& duplicatedMap) = 0;
+    
+    virtual void replaceDependency(KstDataObjectPtr oldObject, KstDataObjectPtr newObject);
+    virtual void replaceDependency(KstVectorPtr oldVector, KstVectorPtr newVector);
+    virtual void replaceDependency(KstMatrixPtr oldMatrix, KstMatrixPtr newMatrix);
 
     virtual bool uses(KstObjectPtr p) const;
 
@@ -78,7 +99,8 @@ class KstDataObject : public KstObject {
     virtual void _showDialog() = 0;
 
   protected:
-    double *vectorRealloced(KstVectorPtr v, double *memptr, int newSize, bool bNew) const;
+    
+    double *vectorRealloced(KstVectorPtr v, double *memptr, int newSize) const;
 
     KstVectorMap _inputVectors;
     KstVectorMap _outputVectors;
@@ -86,18 +108,18 @@ class KstDataObject : public KstObject {
     KstScalarMap _outputScalars;
     KstStringMap _inputStrings;
     KstStringMap _outputStrings;
+    KstMatrixMap _inputMatrices;
+    KstMatrixMap _outputMatrices;
 
-    QString _typeString;
+    QString _typeString, _type;
 
     QValueList<QPair<QString,QString> > _inputVectorLoadQueue;
     QValueList<QPair<QString,QString> > _inputScalarLoadQueue;
     QValueList<QPair<QString,QString> > _inputStringLoadQueue;
+    QValueList<QPair<QString,QString> > _inputMatrixLoadQueue;
     KstCurveHintList *_curveHints;
 };
 
-typedef KstSharedPtr<KstDataObject> KstDataObjectPtr;
-typedef KstObjectList<KstDataObjectPtr> KstDataObjectList;
-
 
 #endif
 // vim: ts=2 sw=2 et
