@@ -33,7 +33,7 @@
 
 // include files for KDE
 #include <dcopclient.h>
-#include <kdebug.h>
+#include "ksdebug.h"
 #include <kdeversion.h>
 #include <kfiledialog.h>
 #include <kmessagebox.h>
@@ -42,10 +42,15 @@
 #include <ksavefile.h>
 
 // application specific includes
+#include "kst2dplot.h"
+#include "kstcsd.h"
 #include "kstdoc.h"
 #include "kstgraphfiledialog_i.h"
 #include "kstequation.h"
+#include "ksteventmonitorentry.h"
 #include "ksthistogram.h"
+#include "kstimage.h"
+#include "kstmatrixdefaults.h"
 #include "kstplugin.h"
 #include "kstpsd.h"
 #include "kstrvector.h"
@@ -53,6 +58,9 @@
 #include "kstvectordefaults.h"
 #include "kstsvector.h"
 #include "kstavector.h"
+#include "kstrmatrix.h"
+#include "kstamatrix.h"
+#include "kstsmatrix.h"
 #include "kstviewwindow.h"
 #include "logevents.h"
 #include "threadevents.h"
@@ -65,6 +73,7 @@ KstDoc::KstDoc(QWidget *parent, const char *name)
   _updating = false;
   _stopping = false;
   _nextEventPaint = false;
+  _title = "Untitled";
 }
 
 KstDoc::~KstDoc() {
@@ -83,6 +92,14 @@ const QString& KstDoc::absFilePath() const {
   return _absFilePath;
 }
 
+void KstDoc::setLastFilePath(const QString& filename) {
+  _lastFilePath = filename;
+}
+
+const QString& KstDoc::lastFilePath() const {
+  return _lastFilePath;
+}
+
 void KstDoc::setTitle(const QString& t) {
   _title = t;
 }
@@ -100,7 +117,7 @@ bool KstDoc::saveModified(bool doDelete) {
       int want_save = KMessageBox::warningYesNoCancel( win, i18n("The current plot definition has been modified. Do you want to save it?"), i18n("Question"));
       switch (want_save) {
         case KMessageBox::Yes:
-          if (_title == i18n("Untitled")) {
+          if (_title == "Untitled") {
             if (!win->slotFileSaveAs()) {
               return false;
             }
@@ -146,7 +163,8 @@ bool KstDoc::newDocument() {
     deleteContents();
     _modified = false;
     _absFilePath = QDir::homeDirPath();
-    _title = i18n("Untitled");
+    _title = "Untitled";
+    KstApp::inst()->newWindow(i18n("default name of first window", "W1"));
     createScalars();
     emit updateDialogs();
   //}
@@ -162,6 +180,7 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
     return false;
   }
 
+  QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
   opening = true;
 
   deleteContents();
@@ -170,34 +189,38 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
 
   QFile f(url.path());
   if (!f.exists()) {
-    KMessageBox::sorry(0L, i18n("%1: There is no file with that name to open.").arg(url.path()));
+    KMessageBox::sorry(KstApp::inst(), i18n("%1: There is no file with that name to open.").arg(url.path()));
     opening = false;
     _updating = false;
     KstApp::inst()->setPaused(false);
+    QApplication::restoreOverrideCursor();
     return false;
   }
 
   _title = url.fileName(false);
   _absFilePath = url.path();
+  _lastFilePath = url.path();
   if (_title.isEmpty()) {
     _title = _absFilePath;
   }
   QDomDocument doc(_title);
 
   if (!f.open(IO_ReadOnly)) {
-    KMessageBox::sorry(0L, i18n("%1: File exists, but kst could not open it.").arg(url.path()));
+    KMessageBox::sorry(KstApp::inst(), i18n("%1: File exists, but kst could not open it.").arg(url.path()));
     opening = false;
     _updating = false;
     KstApp::inst()->setPaused(false);
+    QApplication::restoreOverrideCursor();
     return false;
   }
 
   if (!doc.setContent(&f)) {
-    KMessageBox::sorry(0L, i18n("%1: Not a valid kst plot specification file.").arg(url.path()));
+    KMessageBox::sorry(KstApp::inst(), i18n("%1: Not a valid kst plot specification file.").arg(url.path()));
     f.close();
     opening = false;
     _updating = false;
     KstApp::inst()->setPaused(false);
+    QApplication::restoreOverrideCursor();
     return false;
   }
 
@@ -214,18 +237,24 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
   QString readingDocument = i18n("Reading Kst file");
 
   if (docElem.tagName() != "kstdoc") {
-    f.close();
+    QString err = i18n("Error opening file %1.  Does not appear to be a Kst file.").arg(url.path());
+    KstDebug::self()->log(err, KstDebug::Error);
+    KMessageBox::sorry(KstApp::inst(), err);
     opening = false;
     _updating = false;
     KstApp::inst()->setPaused(false);
+    QApplication::restoreOverrideCursor();
     return false;
   }
 
-  if (!docElem.attribute("version").isEmpty()) {
-    f.close();
+  if (docElem.attribute("version") != "1.2" && !docElem.attribute("version").isEmpty()) {
+    QString err = i18n("Error opening file %2.  Version %1 is too new.  Update Kst or fix the Kst file.").arg(docElem.attribute("version")).arg(url.path());
+    KstDebug::self()->log(err, KstDebug::Error);
+    KMessageBox::sorry(KstApp::inst(), err);
     opening = false;
     _updating = false;
     KstApp::inst()->setPaused(false);
+    QApplication::restoreOverrideCursor();
     return false;
   }
 
@@ -340,6 +369,10 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         KstDataObjectPtr p = new KstPSD(e);
         KstWriteLocker dowl(&KST::dataObjectList.lock());
         KST::dataObjectList.append(p);
+      } else if (e.tagName() == "csdobject") {
+        KstDataObjectPtr p = new KstCSD(e);
+        KstWriteLocker dowl(&KST::dataObjectList.lock());
+        KST::dataObjectList.append(p);
       } else if (e.tagName() == "histogram") {
         KstDataObjectPtr p = new KstHistogram(e);
         KstWriteLocker dowl(&KST::dataObjectList.lock());
@@ -351,11 +384,16 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
       } else if (e.tagName() == "plot") {
         KstBaseCurveList l = kstObjectSubList<KstDataObject,KstBaseCurve>(KST::dataObjectList);
         Kst2DPlotPtr plot = new Kst2DPlot(e);
-        app->plotHolderWhileOpeningDocument().insert(plot->tagName(), plot);
-      } else if (e.tagName() == "matrix") {
-        KstDataObjectPtr p = new KstMatrix(e);
-        KstWriteLocker dowl(&KST::dataObjectList.lock());
-        KST::dataObjectList.append(p);
+        app->plotHolderWhileOpeningDocument()->insert(plot->tagName(), plot);
+      } else if (e.tagName() == "amatrix") {
+        KstAMatrixPtr p = new KstAMatrix(e);
+        KST::addMatrixToList(KstMatrixPtr(p));
+      } else if (e.tagName() == "smatrix") {
+        KstSMatrixPtr p = new KstSMatrix(e);
+        KST::addMatrixToList(KstMatrixPtr(p));
+      } else if (e.tagName() == "rmatrix") {
+        KstRMatrixPtr p = new KstRMatrix(e);
+        KST::addMatrixToList(KstMatrixPtr(p));
       } else if (e.tagName() == "image") {
         KstDataObjectPtr p = new KstImage(e);
         KstWriteLocker dowl(&KST::dataObjectList.lock());
@@ -381,7 +419,7 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
           app->closeWindow(toDelete);
         }
       } else {
-        KstDebug::self()->log(i18n("Unsupported element '%1' in file %2.").arg(e.tagName()).arg(o_file), KstDebug::Warning);
+        KstDebug::self()->log(i18n("Unsupported element '%1' in file %2.").arg(e.tagName()).arg(url.path()), KstDebug::Warning);
       }
     }
     handled++;
@@ -391,14 +429,16 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
   }
 
   if (warnOldKstFile) {
-    KMessageBox::sorry(0L, i18n("You tried to load an old Kst file.  Curves created by equations or PSDs will not be loaded."));
+    QApplication::restoreOverrideCursor();
+    KMessageBox::sorry(KstApp::inst(), i18n("You tried to load an old Kst file.  Curves created by equations or PSDs will not be loaded."));
+    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
   }
 
   app->slotUpdateProgress(0, 0, i18n("Creating plots"));
   // if we have anything left in plotHolderWhileOpeningDocument then
   //  we are most likely reading an old style document, so we create
   //  a default view and fill it with whatever is left...
-  if (!app->plotHolderWhileOpeningDocument().isEmpty() && !app->activeWindow()) {
+  if (!app->plotHolderWhileOpeningDocument()->isEmpty() && !app->activeWindow()) {
 
     QString winName = app->newWindow(QString::null);
     KstViewWindow *win = dynamic_cast<KstViewWindow*>(app->findWindow(winName));
@@ -409,10 +449,11 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         win->view()->setColumns(columns);
       }
 
-      for (Kst2DPlotMap::Iterator it = app->plotHolderWhileOpeningDocument().begin();
-          it != app->plotHolderWhileOpeningDocument().end(); ++it ) {
+      for (Kst2DPlotMap::Iterator it = app->plotHolderWhileOpeningDocument()->begin();
+          it != app->plotHolderWhileOpeningDocument()->end(); ++it ) {
         Kst2DPlotPtr plot = *it;
 
+        // Legacy 1.0 loading code
         if (plot->_width > 0.0 && plot->_height > 0.0 ) {
           win->view()->appendChild(plot.data(), true);
           if (plot->_width > 1.0 && plot->_height > 1.0) {
@@ -429,7 +470,7 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
     }
   }
 
-  app->plotHolderWhileOpeningDocument().clear();
+  app->plotHolderWhileOpeningDocument()->clear();
 
   app->slotUpdateProgress(0, 0, i18n("Loading data"));
 
@@ -441,7 +482,7 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
     KST::dataObjectList.lock().readUnlock();
     for (KstDataObjectList::Iterator i = dol.begin(); i != dol.end(); ++i) {
       assert(*i);
-      //kdDebug() << "Load inputs for " << (*i)->tagName() << " " << (void*)*i << endl;
+      //kstdDebug() << "Load inputs for " << (*i)->tagName() << " " << (void*)*i << endl;
       (*i)->KstRWLock::writeLock();
       bool rc = (*i)->loadInputs();
       (*i)->KstRWLock::writeUnlock();
@@ -465,11 +506,13 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
       names += (*i)->tagName();
     }
 
+    QApplication::restoreOverrideCursor();
 #if KDE_VERSION < KDE_MAKE_VERSION(3,3,90)
-    KMessageBox::informationList(0L, i18n("The Kst file could not be loaded in its entirety due to missing objects or data."), names);
+    KMessageBox::informationList(KstApp::inst(), i18n("The Kst file could not be loaded in its entirety due to missing objects or data."), names);
 #else
-    KMessageBox::errorList(0L, i18n("The Kst file could not be loaded in its entirety due to missing objects or data."), names);
+    KMessageBox::errorList(KstApp::inst(), i18n("The Kst file could not be loaded in its entirety due to missing objects or data."), names);
 #endif
+    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
   }
 
   createScalars();
@@ -497,13 +540,16 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
   opening = false;
 
   KstApp::inst()->setPaused(false);
+  QApplication::restoreOverrideCursor();
+  
   return true;
 }
 
+
 void KstDoc::saveDocument(QTextStream& ts, bool saveAbsoluteVectorPositions) {
   KstApp *app = KstApp::inst();
   ts << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
-  ts << "<kstdoc>" << endl;
+  ts << "<kstdoc version=\"1.2\">" << endl;
   // save window geometry for this kst file
   ts << "  <windowsize>" << endl;
   ts << "    <width>" << app->width() << "</width>" << endl;
@@ -537,17 +583,27 @@ void KstDoc::saveDocument(QTextStream& ts, bool saveAbsoluteVectorPositions) {
   ts << "\" />" << endl;
 
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
+  KstRMatrixList rml = kstObjectSubList<KstMatrix,KstRMatrix>(KST::matrixList);
 
   // save files
   KST::dataSourceList.lock().readLock();
   for (uint i = 0; i < KST::dataSourceList.count(); i++) {
     KstDataSourcePtr dsp = KST::dataSourceList[i];
-    for (KstRVectorList::Iterator it = rvl.begin(); it != rvl.end(); ++it) {
+    bool saved = false;
+    for (KstRVectorList::Iterator it = rvl.begin(); it != rvl.end() && !saved; ++it) {
       if ((*it)->dataSource() == dsp) {
         ts << "  <kstfile>" << endl;
         dsp->save(ts, "    ");
         ts << "  </kstfile>" << endl;
-        break;
+        saved = true;
+      } 
+    }
+    for (KstRMatrixList::Iterator it = rml.begin(); it != rml.end() && !saved; ++it) {
+      if ((*it)->dataSource() == dsp) {
+        ts << "  <kstfile>" << endl;
+        dsp->save(ts, "    ");
+        ts << "  </kstfile>" << endl;
+        saved = true;
       }
     }
   }
@@ -572,18 +628,18 @@ void KstDoc::saveDocument(QTextStream& ts, bool saveAbsoluteVectorPositions) {
   }
 
   // save vectors
-//   for (KstRVectorList::Iterator it = rvl.begin(); it != rvl.end(); ++it) {
-//     ts << "  <vector>" << endl;
-//     (*it)->save(ts, "    ", saveAbsoluteVectorPositions);
-//     ts << "  </vector>" << endl;
-//   }
   for (KstVectorList::Iterator it = KST::vectorList.begin(); it != KST::vectorList.end(); ++it) {
     if ((*it)->saveable()) {
-      //      ts << "  <vector>" << endl;
       (*it)->save(ts, "  ", saveAbsoluteVectorPositions);
-      //ts << "  </vector>" << endl;
     }
   }
+  
+  // save matrices
+  for (KstMatrixList::Iterator it = KST::matrixList.begin(); it != KST::matrixList.end(); ++it) {
+    if ((*it)->saveable()) {
+      (*it)->save(ts, "  ");  
+    }  
+  }
 
   // save data objects
   KST::dataObjectList.lock().readLock();
@@ -591,6 +647,7 @@ void KstDoc::saveDocument(QTextStream& ts, bool saveAbsoluteVectorPositions) {
     (*it)->save(ts, "  ");
   }
   KST::dataObjectList.lock().readUnlock();
+  
   // save plots
   KMdiIterator<KMdiChildView*> *it = app->createIterator();
   while (it->currentItem()) {
@@ -601,7 +658,7 @@ void KstDoc::saveDocument(QTextStream& ts, bool saveAbsoluteVectorPositions) {
         Kst2DPlotPtr plot = kst_cast<Kst2DPlot>(*i);
         if (plot) {
           ts << "  <plot>" << endl;
-          plot->save(ts, "    ");
+          plot->saveAttributes(ts, "    ");
           ts << "  </plot>" << endl;
         }
       }
@@ -628,12 +685,13 @@ void KstDoc::saveDocument(QTextStream& ts, bool saveAbsoluteVectorPositions) {
   ts << "</kstdoc>" << endl;
 }
 
-bool KstDoc::saveDocument(const QString &filename, bool saveAbsoluteVectorPositions) {
+
+bool KstDoc::saveDocument(const QString& filename, bool saveAbsoluteVectorPositions, bool prompt) {
   backupFile(filename);
   QFile f(filename);
 
-  if (f.exists() && filename != absFilePath()) {
-    if (KMessageBox::warningYesNo(0L, i18n("%1: A file of this name already exists.").arg(filename),
+  if (prompt && f.exists() && filename != absFilePath()) {
+    if (KMessageBox::warningYesNo(KstApp::inst(), i18n("%1: A file of this name already exists.").arg(filename),
                                   i18n("Warning"),i18n("&Overwrite"),
                                   i18n("&Cancel")) == KMessageBox::No) {
       return false;
@@ -641,12 +699,18 @@ bool KstDoc::saveDocument(const QString &filename, bool saveAbsoluteVectorPositi
   }
 
   if (!f.open(IO_WriteOnly|IO_Truncate)) {
-    KMessageBox::sorry(0L, i18n("%1: Could not open file for saving. The plot description has not been saved. Try a different filename or directory.").arg(filename));
+    if (prompt) {
+      KMessageBox::sorry(KstApp::inst(), i18n("%1: Could not open file for saving. The plot description has not been saved. Try a different filename or directory.").arg(filename));
+    }
     return false;
   }
 
+  // FIXME: KIO enable
   QTextStream ts(&f);
   ts.setEncoding(QTextStream::UnicodeUTF8);
+  ts.precision(14);
+
+  _lastFilePath = KURL(filename).path();
 
   saveDocument(ts, saveAbsoluteVectorPositions);
 
@@ -659,6 +723,7 @@ bool KstDoc::saveDocument(const QString &filename, bool saveAbsoluteVectorPositi
 
 void KstDoc::deleteContents() {
   KST::vectorDefaults.sync();
+  KST::matrixDefaults.sync();
 
   KstApp *app = KstApp::inst();
   if (app) { // Can be null on application exit
@@ -676,6 +741,10 @@ void KstDoc::deleteContents() {
   KST::vectorList.lock().writeLock();
   KST::vectorList.clear();
   KST::vectorList.lock().writeUnlock();
+  
+  KST::matrixList.lock().writeLock();
+  KST::matrixList.clear();
+  KST::matrixList.lock().writeUnlock();
 
   KST::scalarList.lock().writeLock();
   KST::scalarList.clear();
@@ -696,72 +765,80 @@ void KstDoc::deleteContents() {
   emit updateDialogs();
 }
 
-void KstDoc::samplesUp() {
-  KstRVectorPtr V;
-  int f0, n, skip;
-  bool doSkip, doAve;
-  int fileN;
 
+void KstDoc::samplesUp() {
+  bool changed = false;
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   for (int i = 0; i < (int)rvl.count(); i++) {
-    V = rvl[i];
+    KstRVectorPtr V = rvl[i];
     V->writeLock();
-    f0 = V->reqStartFrame();
-    n = V->reqNumFrames();
-    skip = V->skip();
-    doSkip = V->doSkip();
-    doAve =  V->doAve();
-    fileN = V->fileLength();
+    int f0 = V->reqStartFrame();
+    int n = V->reqNumFrames();
+    int skip = V->skip();
+    bool doSkip = V->doSkip();
+    bool doAve = V->doAve();
+    int fileN = V->fileLength();
 
+    bool didChange = false;
     if (f0 + 2 * n > fileN) {
+      didChange = f0 != fileN - n;
+      changed = changed || didChange;
       f0 = fileN - n;
     } else {
+      didChange = true;
       f0 += n;
     }
-    V->changeFrames(f0, n, skip, doSkip, doAve);
+
+    if (didChange) {
+      V->changeFrames(f0, n, skip, doSkip, doAve);
+    }
     V->writeUnlock();
   }
 
-  setModified();
-  forceUpdate();
-
-  emit dataChanged();
+  if (changed) {
+    setModified();
+    forceUpdate();
+    emit dataChanged();
+  }
 }
 
 
 void KstDoc::samplesDown() {
-  KstRVectorPtr V;
-  int f0, n, skip;
-  bool doSkip, doAve;
-  int fileN;
-
+  bool changed = false;
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   for (int i = 0; i < (int)rvl.count(); i++) {
-    V = rvl[i];
+    KstRVectorPtr V = rvl[i];
     V->writeLock();
-    f0 = V->reqStartFrame();
-    if (f0 == -1)
+    int f0 = V->reqStartFrame();
+    if (f0 == -1) {
       f0 = V->startFrame();
-    n = V->reqNumFrames();
-    skip = V->skip();
-    doSkip = V->doSkip();
-    doAve =  V->doAve();
-    fileN = V->fileLength();
+    }
+    int n = V->reqNumFrames();
+    int skip = V->skip();
+    bool doSkip = V->doSkip();
+    bool doAve =  V->doAve();
 
+    bool didChange = false;
     if (f0 - n < 0) {
+      didChange = f0 != 0;
+      changed = changed || didChange;
       f0 = 0;
     } else {
+      didChange = true;
       f0 -= n;
     }
 
-    V->changeFrames(f0, n, skip, doSkip, doAve);
+    if (didChange) {
+      V->changeFrames(f0, n, skip, doSkip, doAve);
+    }
     V->writeUnlock();
   }
 
-  setModified();
-  forceUpdate();
-
-  emit dataChanged();
+  if (changed) {
+    setModified();
+    forceUpdate();
+    emit dataChanged();
+  }
 }
 
 
@@ -803,48 +880,97 @@ RemoveStatus KstDoc::removeDataObject(const QString& tag) {
 void KstDoc::purge() {
   QString purging = i18n("Purging unused objects");
   bool modified = false;
-  KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
+  bool again = true;
   KstApp *app = KstApp::inst();
 
-  KST::dataObjectList.lock().writeLock();
-  int cnt = rvl.count() + KST::dataObjectList.count();
-  int prg = 0;
-
-  app->slotUpdateProgress(cnt, prg, purging);
-
-  // ASSUMPTION: this only gets called from the data manager!
-  for (KstDataObjectList::Iterator it = KST::dataObjectList.begin(); it != KST::dataObjectList.end(); ++it) {
-    //kdDebug() << "OBJECT: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
-    if ((*it)->getUsage() == 0) {
-      //kdDebug() << "    -> REMOVED" << endl;
-      KstDataObjectList::Iterator byebye = it;
-      --it;
-      KST::dataObjectList.remove(byebye);
-      modified = true;
-    }
-    prg++;
+  while (again) {
+    KST::dataObjectList.lock().readLock();
+    KST::matrixList.lock().readLock();
+    KST::vectorList.lock().readLock();
+    int cnt = KST::matrixList.count() + KST::vectorList.count() + KST::dataObjectList.count();
+    KST::vectorList.lock().readUnlock();
+    KST::matrixList.lock().readUnlock();
+    KST::dataObjectList.lock().readUnlock();
+    
+    int prg = 0;
     app->slotUpdateProgress(cnt, prg, purging);
-  }
-  KST::dataObjectList.lock().writeUnlock();
+    again = false;
+  
+    // ASSUMPTION: this only gets called from the data manager!
+    KST::dataObjectList.lock().writeLock();
+    for (KstDataObjectList::Iterator it = KST::dataObjectList.begin(); it != KST::dataObjectList.end(); ++it) {
+      //kstdDebug() << "OBJECT: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
+      if ((*it)->getUsage() == 0) {
+        //kstdDebug() << "    -> REMOVED" << endl;
+        KstDataObjectList::Iterator byebye = it;
+        --it;
+        KST::dataObjectList.remove(byebye);
+        again = true;
+        modified = true;
+      }
+      ++prg;
+      app->slotUpdateProgress(cnt, prg, purging);
+    }
+    KST::dataObjectList.lock().writeUnlock();
+  
+    KST::vectorList.lock().readLock();
+    KstVectorList vectorList = QDeepCopy<KstVectorList>(KST::vectorList);
+    KST::vectorList.lock().readUnlock();
+    
+    // clear unused vectors that are editable 
+    for (KstVectorList::ConstIterator it = vectorList.begin(); it != vectorList.end(); ++it) {
+      //kstdDebug() << "VECTOR: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
+      if ((*it)->getUsage() == 1) {
+        //kstdDebug() << "    -> REMOVED" << endl;
+        KST::vectorList.lock().writeLock();
+        KST::vectorList.remove(const_cast<KstVector*>((*it).data()));
+        KST::vectorList.lock().writeUnlock();
+        again = true;
+        modified = true;
+      }
+      ++prg;
+      app->slotUpdateProgress(cnt, prg, purging);
+    }
+    
+    KST::matrixList.lock().readLock();
+    KstMatrixList matrixList = QDeepCopy<KstMatrixList>(KST::matrixList);
+    KST::matrixList.lock().readUnlock();
 
-  for (KstRVectorList::Iterator it = rvl.begin(); it != rvl.end(); ++it) {
-    //kdDebug() << "VECTOR: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
-    if ((*it)->getUsage() == 1) {
-      //kdDebug() << "    -> REMOVED" << endl;
-      KST::vectorList.lock().writeLock();
-      KST::vectorList.remove((*it).data());
-      KST::vectorList.lock().writeUnlock();
-      modified = true;
+    // clear unused matrices that are editable
+    for (KstMatrixList::ConstIterator it = matrixList.begin(); it != matrixList.end(); ++it) {
+      if ((*it)->getUsage() == 1) {
+        //kstdDebug() << "    -> REMOVED" << endl;
+        KST::matrixList.lock().writeLock();
+        KST::matrixList.remove(const_cast<KstMatrix*>((*it).data()));
+        KST::matrixList.lock().writeUnlock();
+        again = true;
+        modified = true;
+      }
+      ++prg;
+      app->slotUpdateProgress(cnt, prg, purging);
     }
-    prg++;
-    app->slotUpdateProgress(cnt, prg, purging);
   }
 
-  app->slotUpdateProgress(0, 0, QString::null);
-
-  rvl.clear();
+  KstDataSourceList dataList;
+  KST::dataSourceList.lock().readLock();
+  for (KstDataSourceList::ConstIterator it = KST::dataSourceList.begin(); it != KST::dataSourceList.end(); ++it) {
+      if ((*it)->getUsage() == 1) {
+        //kstdDebug() << "    -> REMOVED" << endl;
+        dataList.append(const_cast<KstDataSource*>((*it).data()));
+        modified = true;
+      }
+  }
+  KST::dataSourceList.lock().readUnlock();
+  KST::dataSourceList.lock().writeLock();
+  for (KstDataSourceList::ConstIterator it = dataList.begin(); it != dataList.end(); ++it) {
+    KST::dataSourceList.remove(const_cast<KstDataSource*>((*it).data()));
+  }
+  KST::dataSourceList.lock().writeUnlock();
+ 
   setModified(modified);
   emit updateDialogs();
+  
+  app->slotUpdateProgress(0, 0, QString::null);
 }
 
 
@@ -867,8 +993,8 @@ bool KstDoc::event(QEvent *e) {
     switch (te->_eventType) {
       case ThreadEvent::UpdateDataDialogs:
         {
-          //kdDebug() << "Update data dialogs" << endl;
           emit dataChanged();
+          KstApp::inst()->updateDataNotifier();
           // HACK: remove me later
           KMdiIterator<KMdiChildView*> *it = KstApp::inst()->createIterator();
           if (it) {
@@ -901,11 +1027,11 @@ bool KstDoc::event(QEvent *e) {
             }
             KstApp::inst()->deleteIterator(it);
           }
-          KstApp::inst()->paintAll();
+          KstApp::inst()->paintAll(KstPainter::P_UPDATE);
         }
         break;
       case ThreadEvent::UpdateAllDialogs:
-        //kdDebug() << "Update ALL dialogs" << endl;
+        //kstdDebug() << "Update ALL dialogs" << endl;
         {
           KMdiIterator<KMdiChildView*>* it = KstApp::inst()->createIterator();
           if (it) {
@@ -946,6 +1072,9 @@ bool KstDoc::event(QEvent *e) {
       switch (le->_eventType) {
         case LogEvent::LogAdded:
           emit logAdded(le->_msg);
+          if (le->_msg.level == KstDebug::Error) {
+            QTimer::singleShot(0, KstApp::inst(), SLOT(createDebugNotifier()));
+          }
           break;
         case LogEvent::LogCleared:
           emit logCleared();
