@@ -24,15 +24,16 @@
 #include <qwhatsthis.h>
 
 // include files for KDE
-#include <kdebug.h>
 #include <kmessagebox.h>
 
 // application specific includes
 #include "curveappearancewidget.h"
+#include "kst2dplot.h"
 #include "kstfilterdialog_i.h"
 #include "kstvcurve.h"
 #include "kstviewwindow.h"
 #include "plugincollection.h"
+#include "plugindialogwidget.h"
 #include "pluginmanager.h"
 #include "scalarselector.h"
 #include "stringselector.h"
@@ -49,10 +50,9 @@ KstFilterDialogI *KstFilterDialogI::globalInstance() {
 }
 
 
-KstFilterDialogI::KstFilterDialogI(QWidget* parent, const char* name,
-                             bool modal, WFlags fl)
+KstFilterDialogI::KstFilterDialogI(QWidget* parent, const char* name, bool modal, WFlags fl)
 : KstPluginDialogI(parent, name, modal, fl) {
-  _curveAppearance->show();
+  _w->_curveAppearance->show();
 }
 
 
@@ -63,39 +63,40 @@ KstFilterDialogI::~KstFilterDialogI() {
 void KstFilterDialogI::show_setCurve(const QString& curveName,
                                   const QString& plotName,
                                   const QString& window) {
-  KstBaseCurveList curves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
 
+  KstBaseCurveList curves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
+  KstVCurveList vcurves = kstObjectSubList<KstBaseCurve, KstVCurve>(curves);
   _window   = window;
   _plotName = plotName;
   _curve    = curveName;
 
   // it should be impossible for the curve not to exist so this should
-  // always be true.  If it is false, we do not properly take care of it, 
+  // always be true.  If it is false, we do not properly take care of it,
   // here and bad things will happen....
-  KstBaseCurvePtr curve = *curves.findTag(curveName);
+  KstVCurvePtr curve = *vcurves.findTag(curveName);
   if (curve) {
     curve->readLock();
     _xvector = curve->xVTag();
     _yvector = curve->yVTag();
     curve->readUnlock();
   }
-  show_New();
+  show();
 }
 
 
 void KstFilterDialogI::updatePluginList() {
   PluginCollection *pc = PluginCollection::self();
   const QMap<QString,Plugin::Data>& _pluginMap = pc->pluginList();
-  QString previous = _pluginList[PluginCombo->currentItem()];
+  QString previous = _pluginList[_w->PluginCombo->currentItem()];
   int newFocus = -1;
   int cnt = 0;
 
-  PluginCombo->clear();
+  _w->PluginCombo->clear();
   _pluginList.clear();
   for (QMap<QString,Plugin::Data>::ConstIterator it = _pluginMap.begin(); it != _pluginMap.end(); ++it) {
     if (it.data()._isFilter) {
       _pluginList += it.data()._name;
-      PluginCombo->insertItem(i18n("%1 (v%2) - %3").arg(it.data()._readableName)
+      _w->PluginCombo->insertItem(i18n("%1 (v%2) - %3").arg(it.data()._readableName)
                               .arg(it.data()._version)
                               .arg(it.data()._description));
       if (it.data()._name == previous) {
@@ -106,9 +107,9 @@ void KstFilterDialogI::updatePluginList() {
   }
 
   if (newFocus != -1) {
-    PluginCombo->setCurrentItem(newFocus);
+    _w->PluginCombo->setCurrentItem(newFocus);
   } else {
-    PluginCombo->setCurrentItem(0);
+    _w->PluginCombo->setCurrentItem(0);
     pluginChanged(0);
   }
 }
@@ -131,7 +132,7 @@ bool KstFilterDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
         }
         plugin->inputVectors().insert((*it)._name, v);
       } else {
-        QObject *field = _pluginInputOutputFrame->child((*it)._name.latin1(), "VectorSelector");
+        QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "VectorSelector");
         if (field) {
           VectorSelector *vs = static_cast<VectorSelector*>(field);
           KstVectorPtr v = *KST::vectorList.findTag(vs->selectedVector());
@@ -145,7 +146,7 @@ bool KstFilterDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
         }
       }
     } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
-      QObject *field = _pluginInputOutputFrame->child((*it)._name.latin1(), "StringSelector");
+      QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "StringSelector");
       if (field) {
         StringSelector *ss = static_cast<StringSelector*>(field);
         KstStringPtr s = *KST::stringList.findTag(ss->selectedString());
@@ -168,7 +169,7 @@ bool KstFilterDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
     } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
       // Nothing
     } else {
-      QObject *field = _pluginInputOutputFrame->child((*it)._name.latin1(), "ScalarSelector");
+      QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "ScalarSelector");
       if (field) {
         ScalarSelector *ss = static_cast<ScalarSelector*>(field);
         KstScalarPtr s = *KST::scalarList.findTag(ss->selectedScalar());
@@ -211,67 +212,59 @@ bool KstFilterDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
 bool KstFilterDialogI::createCurve(KstPluginPtr plugin) {
   KstVectorPtr xVector;
   KstVectorPtr yVector;
-  KstBaseCurveList curves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
-  bool rc = false;
-
-  // Generate new fit name
-  QString c_name = KST::suggestCurveName(plugin->tagName(), true);
 
+  KST::vectorList.lock().readLock();
   KstVectorList::Iterator it = KST::vectorList.findTag(_xvector);
   if (it != KST::vectorList.end()) {
     xVector = *it;
   }
+  KST::vectorList.lock().readUnlock();
 
   if (plugin->outputVectors().contains(plugin->plugin()->data()._filterOutputVector)) {
     yVector = plugin->outputVectors()[plugin->plugin()->data()._filterOutputVector];
   }
 
-  if (xVector && yVector) {
-    plugin->setDirty();
-    KstVCurvePtr fit = new KstVCurve(c_name, KstVectorPtr(xVector), KstVectorPtr(yVector), KstVectorPtr(0L), KstVectorPtr(0L), KstVectorPtr(0L), KstVectorPtr(0L), _curveAppearance->color());
+  if (!xVector || !yVector) {
+    return false;
+  }
+  plugin->setDirty();
+  QString c_name = KST::suggestCurveName(plugin->tagName(), true);
+  KstVCurvePtr fit = new KstVCurve(c_name, KstVectorPtr(xVector), KstVectorPtr(yVector), KstVectorPtr(0L), KstVectorPtr(0L), KstVectorPtr(0L), KstVectorPtr(0L), _w->_curveAppearance->color());
 
-    fit->setHasPoints(_curveAppearance->showPoints());
-    fit->setHasLines(_curveAppearance->showLines());
-    fit->setHasBars(_curveAppearance->showBars());
-    fit->setLineWidth(_curveAppearance->lineWidth());
-    fit->setLineStyle(_curveAppearance->lineStyle());
-    fit->Point.setType(_curveAppearance->pointType());
-    fit->setBarStyle(_curveAppearance->barStyle());
-    fit->setPointDensity(_curveAppearance->pointDensity());
+  fit->setHasPoints(_w->_curveAppearance->showPoints());
+  fit->setHasLines(_w->_curveAppearance->showLines());
+  fit->setHasBars(_w->_curveAppearance->showBars());
+  fit->setLineWidth(_w->_curveAppearance->lineWidth());
+  fit->setLineStyle(_w->_curveAppearance->lineStyle());
+  fit->Point.setType(_w->_curveAppearance->pointType());
+  fit->setBarStyle(_w->_curveAppearance->barStyle());
+  fit->setPointDensity(_w->_curveAppearance->pointDensity());
 
-    //
-    // add the curve to the plot...
-    //
-    KstViewWindow *w = dynamic_cast<KstViewWindow*>(KstApp::inst()->findWindow(_window));
-    if (w && w->view()->findChild(_plotName)) {
-      Kst2DPlotPtr plot = kst_cast<Kst2DPlot>(w->view()->findChild(_plotName));
-      if (plot) {
-        plot->addCurve(fit.data());
-        w->view()->paint(P_PAINT);
-      }
+  KstViewWindow *w = dynamic_cast<KstViewWindow*>(KstApp::inst()->findWindow(_window));
+  if (w && w->view()->findChild(_plotName)) {
+    Kst2DPlotPtr plot = kst_cast<Kst2DPlot>(w->view()->findChild(_plotName));
+    if (plot) {
+      plot->addCurve(fit.data());
     }
-
-    KST::dataObjectList.lock().writeLock();
-    KST::dataObjectList.append(fit.data());
-    KST::dataObjectList.lock().writeUnlock();
-
-    rc = true;
   }
 
-  return rc;
+  KST::dataObjectList.lock().writeLock();
+  KST::dataObjectList.append(fit.data());
+  KST::dataObjectList.lock().writeUnlock();
+  return true;
 }
 
 
-bool KstFilterDialogI::new_I() {
+bool KstFilterDialogI::newObject() {
   QString tagName = _tagName->text();
 
-  if (KST::dataTagNameNotUnique(tagName, true, this)) {
+  if (KstData::self()->dataTagNameNotUnique(tagName, true, this)) {
     _tagName->setFocus();
     return false;
   } else {
-    int pitem = PluginCombo->currentItem();
+    int pitem = _w->PluginCombo->currentItem();
 
-    if (pitem >= 0 && PluginCombo->count() > 0) {
+    if (pitem >= 0 && _w->PluginCombo->count() > 0) {
       KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[pitem]);
 
       if (pPtr) {
@@ -289,14 +282,14 @@ bool KstFilterDialogI::new_I() {
           if (saveOutputs(plugin, pPtr)) {
             if (plugin->isValid()) {
               if (!createCurve(plugin)) {
-                KMessageBox::sorry(this, i18n("There is an error in the plugin you entered. Please fix it."));
+                KMessageBox::sorry(this, i18n("There is an error in the plugin you entered."));
               } else {
                 KST::dataObjectList.lock().writeLock();
                 KST::dataObjectList.append(plugin.data());
                 KST::dataObjectList.lock().writeUnlock();
               }
             } else {
-              KMessageBox::sorry(this, i18n("There is an error in the plugin you entered. Please fix it."));
+              KMessageBox::sorry(this, i18n("There is an error in the plugin you entered."));
             }
           }
         }
@@ -310,10 +303,10 @@ bool KstFilterDialogI::new_I() {
 
 
 void KstFilterDialogI::generateEntries(bool input, int& cnt, QWidget *parent, QGridLayout *grid, const QValueList<Plugin::Data::IOValue>& table) {
-  
+
   // get fixed vector for filter
   QString fixedVector;
-  const QString& pluginName = _pluginList[PluginCombo->currentItem()];
+  const QString& pluginName = _pluginList[_w->PluginCombo->currentItem()];
   const Plugin::Data& pluginData = PluginCollection::self()->pluginList()[PluginCollection::self()->pluginNameList()[pluginName]];
   if (input) {
     fixedVector = pluginData._filterInputVector;
@@ -420,7 +413,7 @@ void KstFilterDialogI::generateEntries(bool input, int& cnt, QWidget *parent, QG
     grid->addWidget(label, cnt, 0);
     _pluginWidgets.push_back(label);
     label->show();
-    
+
     grid->addWidget(widget, cnt, 1);
     _pluginWidgets.push_back(widget);
     widget->show();
