@@ -18,6 +18,8 @@
 
 #include <assert.h>
 
+#include <stdio.h>
+
 // include files for Qt
 #include <qlineedit.h>
 #include <qtooltip.h>
@@ -25,15 +27,17 @@
 #include <qregexp.h>
 
 // include files for KDE
-#include <kdebug.h>
 #include <kmessagebox.h>
 
 // application specific includes
 #include "curveappearancewidget.h"
+#include "kst2dplot.h"
 #include "kstfitdialog_i.h"
 #include "kstvcurve.h"
+#include "kstviewlabel.h"
 #include "kstviewwindow.h"
 #include "plugincollection.h"
+#include "plugindialogwidget.h"
 #include "pluginmanager.h"
 #include "scalarselector.h"
 #include "stringselector.h"
@@ -49,10 +53,9 @@ KstFitDialogI *KstFitDialogI::globalInstance() {
 }
 
 
-KstFitDialogI::KstFitDialogI(QWidget* parent, const char* name,
-                             bool modal, WFlags fl)
+KstFitDialogI::KstFitDialogI(QWidget* parent, const char* name, bool modal, WFlags fl)
 : KstPluginDialogI(parent, name, modal, fl) {
-  _curveAppearance->show();
+  _w->_curveAppearance->show();
 }
 
 
@@ -63,16 +66,17 @@ KstFitDialogI::~KstFitDialogI() {
 void KstFitDialogI::show_setCurve(const QString& strCurve,
                                   const QString& strPlotName,
                                   const QString& strWindow) {
-                                    
-  KstBaseCurvePtr curve;
+
+  KstVCurvePtr curve;
   KstBaseCurveList curves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
+  KstVCurveList vcurves = kstObjectSubList<KstBaseCurve, KstVCurve>(curves);
   KstPluginList c = kstObjectSubList<KstDataObject, KstPlugin>(KST::dataObjectList);
   QString new_label;
 
   _strWindow   = strWindow;
   _strPlotName = strPlotName;
   _strCurve    = strCurve;
-  curve = *curves.findTag(strCurve);
+  curve = *vcurves.findTag(strCurve);
   if (curve) {
     KstReadLocker rl(curve);
     _yvector = curve->yVTag();
@@ -82,26 +86,25 @@ void KstFitDialogI::show_setCurve(const QString& strCurve,
   if (_xvector == "<None>" || _yvector == "<None>") {
     return;
   }
-  show_New();  
+  show();
 }
 
 
 void KstFitDialogI::updatePluginList() {
   PluginCollection *pc = PluginCollection::self();
   const QMap<QString,Plugin::Data>& _pluginMap = pc->pluginList();
-  QString previous = _pluginList[PluginCombo->currentItem()];
+  QString previous = _pluginList[_w->PluginCombo->currentItem()];
   QMap<QString,Plugin::Data>::ConstIterator it;
   int newFocus = -1;
   int cnt = 0;
 
-  PluginCombo->clear();
+  _w->PluginCombo->clear();
   _pluginList.clear();
   for (it = _pluginMap.begin(); it != _pluginMap.end(); ++it) {
     if (it.data()._isFit) {
       if (!it.data()._isFitWeighted || _evector != "<None>") {
         _pluginList += it.data()._name;
-        PluginCombo->insertItem(i18n("%1 (v%2)").arg(it.data()._readableName)
-                                .arg(it.data()._version));
+        _w->PluginCombo->insertItem(i18n("%1 (v%2)").arg(it.data()._readableName).arg(it.data()._version));
         if (it.data()._name == previous) {
           newFocus = cnt;
         }
@@ -111,9 +114,9 @@ void KstFitDialogI::updatePluginList() {
   }
 
   if (newFocus != -1) {
-    PluginCombo->setCurrentItem(newFocus);
+    _w->PluginCombo->setCurrentItem(newFocus);
   } else {
-    PluginCombo->setCurrentItem(0);
+    _w->PluginCombo->setCurrentItem(0);
     pluginChanged(0);
   }
 }
@@ -122,15 +125,6 @@ void KstFitDialogI::updatePluginList() {
 bool KstFitDialogI::createCurve(KstPluginPtr plugin) {
   KstVectorPtr xVector;
   KstVectorPtr yVector;
-  KstVectorPtr wVector;
-  KstLabel* label;
-  KstBaseCurveList curves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
-  KstVCurvePtr fit;
-  QString strLabel;
-  bool bRetVal = false;
-
-  // Generate new fit name
-  QString c_name = KST::suggestCurveName(plugin->tagName(), true);
 
   if (plugin->inputVectors().contains("X Array")) {
     xVector = plugin->inputVectors()["X Array"];
@@ -139,78 +133,80 @@ bool KstFitDialogI::createCurve(KstPluginPtr plugin) {
     yVector = plugin->outputVectors()["Y Fitted"];
   }
 
-  if (xVector && yVector) {
-    fit = new KstVCurve(c_name, KstVectorPtr(xVector), KstVectorPtr(yVector), KstVectorPtr(0L), KstVectorPtr(0L), KstVectorPtr(0L), KstVectorPtr(0L), _curveAppearance->color());
-    if (fit) {
-      fit->setHasPoints(_curveAppearance->showPoints());
-      fit->setHasLines(_curveAppearance->showLines());
-      fit->setHasBars(_curveAppearance->showBars());
-      fit->setLineWidth(_curveAppearance->lineWidth());
-      fit->setLineStyle(_curveAppearance->lineStyle());
-      fit->Point.setType(_curveAppearance->pointType());
-      fit->setBarStyle(_curveAppearance->barStyle());
-      fit->setPointDensity(_curveAppearance->pointDensity());
+  if (!xVector || !yVector) {
+    return false;
+  }
 
-      // create the necessary label
-      strLabel = i18n("curvename: pluginname\\n", "%2: %1\\n").
-                 arg(_strCurve.replace( '_',  "\\_" ).remove( QRegExp( "^[C][0-9]{1,2}\\-" ) )).
-                 arg(plugin->plugin()->data()._readableName);
+  QString c_name = KST::suggestCurveName(plugin->tagName(), true);
 
-      KstVectorPtr vectorParam = plugin->outputVectors()["Parameters"];
-      if (vectorParam) {
-        plugin->createFitScalars();
-        for (int i = 0; i < vectorParam->length(); i++) {
-          QString strParamName = plugin->plugin()->parameterName(i);
-          if (!strParamName.isEmpty()) {
-            if (plugin->outputScalars().contains(strParamName)) {
-              KstScalarPtr scalar = plugin->outputScalars()[strParamName];
+  KstVCurvePtr fit = new KstVCurve(c_name, KstVectorPtr(xVector), KstVectorPtr(yVector), KstVectorPtr(0L), KstVectorPtr(0L), KstVectorPtr(0L), KstVectorPtr(0L), _w->_curveAppearance->color());
+  fit->setHasPoints(_w->_curveAppearance->showPoints());
+  fit->setHasLines(_w->_curveAppearance->showLines());
+  fit->setHasBars(_w->_curveAppearance->showBars());
+  fit->setLineWidth(_w->_curveAppearance->lineWidth());
+  fit->setLineStyle(_w->_curveAppearance->lineStyle());
+  fit->Point.setType(_w->_curveAppearance->pointType());
+  fit->setBarStyle(_w->_curveAppearance->barStyle());
+  fit->setPointDensity(_w->_curveAppearance->pointDensity());
 
-              if (scalar) {
-                strLabel += i18n("parametername: [scalarname]\\n", "%1: [%2]\\n").arg(strParamName).arg(scalar->tagName());
-              }
-            }
-          }
+  QString strLabel = i18n("curvename: pluginname\\n", "%2: %1\\n").
+    arg(_strCurve.replace('_', "\\_").remove(QRegExp("^[C][0-9]{1,2}\\-"))).
+    arg(plugin->plugin()->data()._readableName);
+
+  if (plugin->outputVectors().contains("Parameters")) {
+    plugin->createFitScalars();
+    int i = 0;
+    int length = (plugin->outputVectors())["Parameters"]->length();
+    QString strParamName;
+    for (strParamName = plugin->plugin()->parameterName(0); 
+        !strParamName.isEmpty() && i < length; 
+         strParamName = plugin->plugin()->parameterName(++i)) {
+      if (plugin->outputScalars().contains(strParamName)) {
+        KstScalarPtr scalar = plugin->outputScalars()[strParamName];
+        if (scalar) {
+          strLabel += i18n("parametername: [scalarname]\\n", "%1: [%2]\\n").arg(strParamName).arg(scalar->tagName());
         }
       }
-
-      //
-      // add the label and the curve to the plot...
-      //
-      KstViewWindow *w = dynamic_cast<KstViewWindow*>(KstApp::inst()->findWindow(_strWindow));
-      if (w && w->view()->findChild(_strPlotName)) {
-        Kst2DPlotPtr plot = kst_cast<Kst2DPlot>(w->view()->findChild(_strPlotName));
-        if (plot) {
-          label = new KstLabel(strLabel, LxTy, 0.0, 0.05, 0.05);
-          plot->addLabel(label);
-          plot->addCurve(fit.data());
-        }
-      }
-
-      KST::dataObjectList.lock().writeLock();
-      KST::dataObjectList.append(fit.data());
-      KST::dataObjectList.lock().writeUnlock();
-
-      bRetVal = true;
     }
   }
 
-  return bRetVal;
+  // add the label and the curve to the plot
+  KstViewWindow *w = dynamic_cast<KstViewWindow*>(KstApp::inst()->findWindow(_strWindow));
+  if (w && w->view()->findChild(_strPlotName)) {
+    Kst2DPlotPtr plot = kst_cast<Kst2DPlot>(w->view()->findChild(_strPlotName));
+    if (plot) {
+      KstViewLabelPtr label = new KstViewLabel(strLabel, KST_JUSTIFY_H_LEFT | KST_JUSTIFY_V_TOP, 0.0);
+      label->setTransparent(true);
+      plot->appendChild(KstViewObjectPtr(label), true);
+      label->resizeFromAspect(0.1, 0.1, 0.05, 0.05);
+      label->setJustification(SET_KST_JUSTIFY(KST_JUSTIFY_H_LEFT, KST_JUSTIFY_V_BOTTOM));
+      label->setAutoResize(true);
+
+      plot->addCurve(KstBaseCurvePtr(fit));
+    }
+  }
+
+  KST::dataObjectList.lock().writeLock();
+  KST::dataObjectList.append(fit.data());
+  KST::dataObjectList.lock().writeUnlock();
+
+  return true;
 }
 
 
-bool KstFitDialogI::new_I() {
+bool KstFitDialogI::newObject() {
   QString tagName = _tagName->text();
 
   if (tagName != plugin_defaultTag) {
-    if (KST::dataTagNameNotUnique(tagName, true, this)) {
+    if (KstData::self()->dataTagNameNotUnique(tagName, true, this)) {
       _tagName->setFocus();
       return false;
     }
   }
 
-  int pitem = PluginCombo->currentItem();
+  int pitem = _w->PluginCombo->currentItem();
 
-  if (pitem >= 0 && PluginCombo->count() > 0) {
+  if (pitem >= 0 && _w->PluginCombo->count() > 0) {
     KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[pitem]);
 
     if (pPtr) {
@@ -260,14 +256,14 @@ bool KstFitDialogI::new_I() {
 
 
 void KstFitDialogI::generateEntries(bool input, int& cnt, QWidget *parent, QGridLayout *grid, const QValueList<Plugin::Data::IOValue>& table) {
-  
-  const QString& pluginName = _pluginList[PluginCombo->currentItem()];
+
+  const QString& pluginName = _pluginList[_w->PluginCombo->currentItem()];
   const Plugin::Data& pluginData = PluginCollection::self()->pluginList()[PluginCollection::self()->pluginNameList()[pluginName]];
   bool isWeighted = pluginData._isFitWeighted;
-  
+
   QString scalarLabelTemplate, vectorLabelTemplate, stringLabelTemplate;
   int iInputVector = 0;
-  
+
   if (input) {
     stringLabelTemplate = i18n("Input String - %1:");
     scalarLabelTemplate = i18n("Input Scalar - %1:");
@@ -361,7 +357,7 @@ void KstFitDialogI::generateEntries(bool input, int& cnt, QWidget *parent, QGrid
         } else {
           widget = new VectorSelector(parent, (*it)._name.latin1());
           connect(widget, SIGNAL(newVectorCreated(const QString&)), this, SIGNAL(modified()));
-        } 
+        }
       }
     } else {
       widget = new QLineEdit(parent, (*it)._name.latin1());
@@ -370,11 +366,11 @@ void KstFitDialogI::generateEntries(bool input, int& cnt, QWidget *parent, QGrid
     grid->addWidget(label, cnt, 0);
     _pluginWidgets.push_back(label);
     label->show();
-    
+
     grid->addWidget(widget, cnt, 1);
     _pluginWidgets.push_back(widget);
     widget->show();
-    
+
     if (!(*it)._description.isEmpty()) {
       QWhatsThis::remove(label);
       QWhatsThis::remove(widget);
@@ -389,17 +385,17 @@ void KstFitDialogI::generateEntries(bool input, int& cnt, QWidget *parent, QGrid
 
 bool KstFitDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
   bool rc = true;
-  
+
   KST::vectorList.lock().readLock();
   KST::scalarList.lock().readLock();
   KST::stringList.lock().readLock();
   const QValueList<Plugin::Data::IOValue>& itable = p->data()._inputs;
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
     if ((*it)._type == Plugin::Data::IOValue::TableType) {
-      QObject *field = _pluginInputOutputFrame->child((*it)._name.latin1(), "VectorSelector");
+      QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "VectorSelector");
       KstVectorPtr v;
       if (!field) {
-        field = _pluginInputOutputFrame->child((*it)._name.latin1(), "QLabel");
+        field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "QLabel");
         assert(field);
         v = *KST::vectorList.findTag(static_cast<QLabel*>(field)->text());
       } else {
@@ -417,7 +413,7 @@ bool KstFitDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
         rc = false;
       }
     } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
-      QObject *field = _pluginInputOutputFrame->child((*it)._name.latin1(), "StringSelector");
+      QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "StringSelector");
       assert(field);
       StringSelector *ss = static_cast<StringSelector*>(field);
       KstStringPtr s = *KST::stringList.findTag(ss->selectedString());
@@ -439,7 +435,7 @@ bool KstFitDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
     } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
       // Nothing
     } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
-      QObject *field = _pluginInputOutputFrame->child((*it)._name.latin1(), "ScalarSelector");
+      QObject *field = _w->_pluginInputOutputFrame->child((*it)._name.latin1(), "ScalarSelector");
       assert(field);
       ScalarSelector *ss = static_cast<ScalarSelector*>(field);
       KstScalarPtr s = *KST::scalarList.findTag(ss->selectedScalar());
@@ -474,7 +470,7 @@ bool KstFitDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
   KST::stringList.lock().readUnlock();
   KST::scalarList.lock().readUnlock();
   KST::vectorList.lock().readUnlock();
-  
+
   return rc;
 }
 
