@@ -2,7 +2,7 @@
                      kstimage.h: image type for kst
                              -------------------
     begin                : Mon July 19 2004
-    copyright            : (C) 2004 by University of British Columbia
+    copyright            : (C) 2005 by University of British Columbia
     email                :
  ***************************************************************************/
 
@@ -21,11 +21,13 @@
 #include <kpalette.h>
 
 #include "kstmatrix.h"
+#include "kstbasecurve.h"
+#include "kst_export.h"
 
 /**A class for handling images for Kst
  *@author University of British Columbia
  */
-class KstImage: public KstDataObject {
+class KST_EXPORT KstImage: public KstBaseCurve {
   public:
     //constructor for colormap only
     KstImage(const QString &in_tag, KstMatrixPtr in_matrix, double lowerZ, double upperZ, bool autoThreshold, KPalette* pal);
@@ -42,20 +44,18 @@ class KstImage: public KstDataObject {
         const QColor& contourColor,
         int contourWeight);
 
-    KstImage(QDomElement& e);
+    KstImage(const QDomElement& e);
     virtual ~KstImage();
 
     virtual void _showDialog();
     virtual void save(QTextStream &ts, const QString& indent = QString::null);
     virtual UpdateType update(int update_counter = -1);
     virtual QString propertyString() const;
-    virtual bool loadInputs();
-
-    virtual int sampleCount() const { return NS; }
-    virtual int samplesPerFrame() const { return 1; }
+    
+    virtual KstCurveType curveType() const;
 
     virtual bool getNearestZ(double x, double y, double& z);
-    virtual QRgb getMappedColor(double x, double y);
+    virtual QColor getMappedColor(double x, double y);
     virtual void setPalette(KPalette* pal);
     virtual void setUpperThreshold(double z);
     virtual void setLowerThreshold(double z);
@@ -66,8 +66,10 @@ class KstImage: public KstDataObject {
     virtual double lowerThreshold() const { return _zLower; }
     virtual bool autoThreshold() const { return _autoThreshold; }
 
-    virtual QString matrixTag() const { return _matrix->tagName(); }
+    virtual QString matrixTag() const;
+    virtual KstMatrixPtr matrix() const;
     virtual QString paletteName() const;
+    virtual KPalette* palette() const { return _pal; }
 
     virtual void matrixDimensions(double &x, double &y, double &width, double &height);
 
@@ -91,17 +93,33 @@ class KstImage: public KstDataObject {
     //other properties
     virtual bool hasContourMap() const { return _hasContourMap; }
     virtual bool hasColorMap() const { return _hasColorMap; }
+    
+    // labels for plots
+    virtual QString xLabel() const;
+    virtual QString yLabel() const;
+    virtual QString topLabel() const;
+    
+    virtual KstDataObjectPtr makeDuplicate(KstDataObjectDataObjectMap& duplicatedMap);
 
-  protected:
-
-    int NS;
-    QValueList<QPair<QString,QString> > _inputMatrixLoadQueue;
+    // see KstBaseCurve::providerDataObject
+    virtual KstDataObjectPtr providerDataObject() const;
+    
+    // see KstBaseCurve::distanceToPoint
+    virtual double distanceToPoint(double xpos, double dx, double ypos) const;
+    
+    // see KstBaseCurve::paint
+    virtual void paint(const KstCurveRenderContext& context);
+    
+    // see KstBaseCurve::yRange
+    virtual void yRange(double xFrom, double xTo, double* yMin, double* yMax);
+    
+    // see KstBaseCurve::paintLegendSymbol
+    virtual void paintLegendSymbol(QPainter *p, const QRect& bound);
 
   private:
     //use these to set defaults when either is not used.
     void setColorDefaults();
     void setContourDefaults();
-    KstMatrixPtr _matrix;
     KPalette* _pal;
     //upper and lower thresholds
     double _zUpper;
