@@ -19,16 +19,13 @@
 #ifndef KSTLINESTYLE_H
 #define KSTLINESTYLE_H
 
-#define KSTLINESTYLE_MAXTYPE 5
+#include "kst_export.h"
 
-static Qt::PenStyle KstLineStyle[KSTLINESTYLE_MAXTYPE] = { 
-	Qt::SolidLine, 
-	Qt::DashLine, 
-	Qt::DotLine, 
-	Qt::DashDotLine, 
-	Qt::DashDotDotLine
-};
+KST_EXPORT extern const unsigned int KSTLINESTYLE_MAXTYPE;
+KST_EXPORT extern Qt::PenStyle KstLineStyle[];
 
 #define KSTPOINTDENSITY_MAXTYPE 4
+#define KSTLINEWIDTH_MAX        5
 
 #endif
+// vim: ts=2 sw=2 et
