@@ -23,20 +23,22 @@
  *@author C. Barth Netterfield
  */
 
-#define KSTPOINT_MAXTYPE 8
+#include "kst_export.h"
 
-class KstPoint {
+#define KSTPOINT_MAXTYPE 13
+
+class KST_EXPORT KstPoint {
   public:
     KstPoint();
     ~KstPoint();
 
     /** draw the point on a painter the scale of the point is based on size */
-    void draw(QPainter *p, int x, int y, int size=-1);
+    void draw(QPainter *p, int x, int y, int lineSize=0, int size=-1);
     /** set the type of the point: arbitrary meaning only */
     void setType(int type);
     /** get the type of the point for storage */
     int type() const;
-    /** Get the dimention of the current point type */
+    /** Get the dimension of the current point type */
     int dim(QPainter *p) const;
   private:
     int Type;
