@@ -35,19 +35,20 @@
 
 extern "C" void rdft(int n, int isgn, double *a);
 
-static const QString& INVECTOR = KGlobal::staticQString("I");
-static const QString& SVECTOR = KGlobal::staticQString("S");
-static const QString& FVECTOR = KGlobal::staticQString("F");
+const QString& KstPSD::INVECTOR = KGlobal::staticQString("I");
+const QString& KstPSD::SVECTOR = KGlobal::staticQString("S");
+const QString& KstPSD::FVECTOR = KGlobal::staticQString("F");
 
 #define KSTPSDMAXLEN 27
 KstPSD::KstPSD(const QString &in_tag, KstVectorPtr in_V,
                          double in_freq, bool in_average, int in_len,
                          bool in_apodize, bool in_removeMean,
-                         const QString &in_VUnits, const QString &in_RUnits)
+                         const QString &in_VUnits, const QString &in_RUnits, int in_apodizeFxn, 
+                         double in_gaussianSigma)
 : KstDataObject() {
   commonConstructor(in_tag, in_V, in_freq, in_average, in_len,
                     in_apodize, in_removeMean,
-                    in_VUnits, in_RUnits);
+                    in_VUnits, in_RUnits, in_apodizeFxn, in_gaussianSigma);
                     
   setDirty();
 }
@@ -64,6 +65,8 @@ KstPSD::KstPSD(const QDomElement &e)
   bool in_average = true;
   bool in_removeMean = true;
   bool in_apodize = true;
+  int in_apodizeFxn = 0;
+  double in_gaussianSigma = 3.0;
   int in_len = 12;
 
   QDomNode n = e.firstChild();
@@ -90,6 +93,10 @@ KstPSD::KstPSD(const QDomElement &e)
         } else {
           in_apodize = true;
         }
+      } else if (e.tagName() == "apodizefxn") {
+        in_apodizeFxn = e.text().toInt();
+      } else if (e.tagName() == "gaussiansigma") {
+        in_gaussianSigma = e.text().toDouble();
       } else if (e.tagName() == "removeMean") {
         if (e.text() == "0") {
           in_removeMean = false;
@@ -108,7 +115,7 @@ KstPSD::KstPSD(const QDomElement &e)
   _inputVectorLoadQueue.append(qMakePair(INVECTOR, vecName));
   commonConstructor(in_tag, in_V, in_freq, in_average, in_len,
                     in_apodize, in_removeMean,
-                    in_VUnits, in_RUnits);
+                    in_VUnits, in_RUnits, in_apodizeFxn, in_gaussianSigma);
 }
 
 
@@ -116,15 +123,19 @@ void KstPSD::commonConstructor(const QString& in_tag, KstVectorPtr in_V,
                                double in_freq, bool in_average, int in_len,
                                bool in_apodize, bool in_removeMean,
                                const QString& in_VUnits,
-                               const QString& in_RUnits) {
+                               const QString& in_RUnits, int in_apodizeFxn, double in_gaussianSigma) {
 
   _typeString = i18n("Power Spectrum");
+  _type = "PowerSpectrum";
   _inputVectors[INVECTOR] = in_V;
   setTagName(in_tag);
   _Freq = in_freq;
   _Average = in_average;
   _Len = in_len;
   _Apodize = in_apodize;
+  _apodizeFxn = in_apodizeFxn;
+  _gaussianSigma = in_gaussianSigma;
+  _prevApodizeFxn = -1;
   _RemoveMean = in_removeMean;
   _vUnits = in_VUnits;
   _rUnits = in_RUnits;
@@ -205,16 +216,88 @@ void KstPSD::GenW(int len) {
   int i;
   double sW = 0;
 
-  if (len != _WLen) {
-    for (i = 0; i < len; i++) {
-      _w[i] = (1 - cos(2*M_PI*double(i)/double(len)));
-      sW += _w[i] * _w[i];
-    }
-    sW = sqrt(sW/double(len));
-    for (i = 0; i < len; i++) {
-      _w[i] /= sW;
+  if (len != _WLen || _prevApodizeFxn != _apodizeFxn) {
+    double a = len/2;
+    double x;
+    
+    switch (_apodizeFxn) {
+      // original apodization function
+      case 0: 
+        for (i = 0; i < len; i++) {
+          _w[i] = (1 - cos(2*M_PI*double(i)/double(len)));
+          sW += _w[i] * _w[i];
+        }
+        sW = sqrt(sW/double(len));
+        for (i = 0; i < len; i++) {
+          _w[i] /= sW;
+        }
+        break;
+      // Bartlett function
+      case 1:
+        for (i = 0; i < len; i++) {
+          x = i-a;
+          _w[i] = (1 - fabs(x)/a);
+        }
+        break;
+      // Blackman function 
+      case 2:
+        for (i = 0; i < len; i++) {
+          x = i-a;
+          _w[i] = 0.42 + 0.5*cos(M_PI*x/a) + 0.08*cos(2*M_PI*x/a);
+        }
+        break;
+      // Connes function
+      case 3: 
+        for (i = 0; i < len; i++) {
+          x = i-a;
+          _w[i] = pow(1-(x*x)/(a*a), 2);
+        }
+        break;
+      // cosine function
+      case 4:
+        for (i = 0; i < len; i++) {
+          x = i-a;
+          _w[i] = cos((M_PI*x)/(2*a));
+        }
+        break;
+      // Gaussian function
+      case 5:
+        for (i = 0; i < len; i++) {
+          x = i-a;
+          _w[i] = exp((-1*x*x)/(2*_gaussianSigma*_gaussianSigma));
+        }
+        break;
+      // Hamming function
+      case 6:
+        for (i = 0; i < len; i++) {
+          x = i-a;
+          _w[i] = 0.54 + 0.46*cos(M_PI*x/a);
+        }  
+        break;
+      // Hanning function
+      case 7:
+        for (i = 0; i < len; i++) {
+          x = i-a;
+          _w[i] = pow(cos((M_PI*x)/(2*a)), 2);
+        }
+        break;
+      // Welch function
+      case 8:
+        for (i = 0; i < len; i++) {
+          x = i-a;
+          _w[i] = 1 - (x*x)/(a*a);
+        }
+        break;
+      // uniform
+      default:
+        for (i = 0; i < len; i++) {
+          _w[i] = 1;
+        }
+        break;
     }
+    // remember last used apodization function and length
     _WLen = len;
+    _prevApodizeFxn = _apodizeFxn;
   }
 }
 
@@ -233,14 +316,9 @@ KstObject::UpdateType KstPSD::update(int update_counter) {
   }
 
   int i_subset, i_samp;
-  int n_subsets;
-  int v_len;
   int copyLen;
-  double *psd, *f;
   double mean;
-  double nf;
   KstVectorPtr iv = _inputVectors[INVECTOR];
-  bool done;
 
   if (update_counter <= 0) {
     assert(update_counter == 0);
@@ -251,30 +329,33 @@ KstObject::UpdateType KstPSD::update(int update_counter) {
 
   _adjustLengths();
 
-  v_len = iv->length();
-  n_subsets = v_len/_PSDLen+1;
-  if (v_len%_PSDLen==0) n_subsets--;
+  int v_len = iv->length();
+  int n_subsets = v_len / _PSDLen + 1;
+  if (v_len % _PSDLen == 0) {
+    --n_subsets;
+  }
 
   _last_n_new += iv->numNew();
+  assert(_last_n_new >= 0);
 
   if ((!xUpdated || (_last_n_new < _PSDLen/16 &&
           n_subsets - _last_n_subsets < 1 &&
-          iv->length() != iv->numNew())) && !force ) {
+          iv->length() != iv->numNew())) && !force) {
     return setLastUpdateResult(NO_CHANGE);
   }
 
-  psd = (*_sVector)->value();
-  f = (*_fVector)->value();
+  double *psd = (*_sVector)->value();
+  double *f = (*_fVector)->value();
 
-  for (i_samp = 0; i_samp < _PSDLen; i_samp++) {
+  for (i_samp = 0; i_samp < _PSDLen; ++i_samp) {
     psd[i_samp] = 0;
-    f[i_samp] = i_samp*0.5*_Freq/( _PSDLen-1 );
+    f[i_samp] = i_samp * 0.5 * _Freq / (_PSDLen - 1);
   }
 
-  nf = 0;
-  done = false;
+  double nf = 0;
+  bool done = false;
 
-  for (i_subset = 0; !done; i_subset++) {
+  for (i_subset = 0; !done; ++i_subset) {
     // copy each chunk into a[] and find mean
     if (i_subset*_PSDLen + _ALen < v_len) {
       copyLen = _ALen;
@@ -284,14 +365,12 @@ KstObject::UpdateType KstPSD::update(int update_counter) {
     }
 
     mean = 0;
-    for (i_samp = 0; i_samp < copyLen; i_samp++) {
-      mean += (
-        _a[i_samp] =
-        iv->interpolate(i_samp + i_subset*_PSDLen, v_len)
-        );
+    for (i_samp = 0; i_samp < copyLen; ++i_samp) {
+      _a[i_samp] = iv->interpolate(i_samp + i_subset*_PSDLen, v_len);
+      mean += _a[i_samp];
     }
     if (copyLen > 1) {
-      mean /= (double)copyLen;
+      mean /= double(copyLen);
     }
 
     if (apodize()) {
@@ -300,32 +379,31 @@ KstObject::UpdateType KstPSD::update(int update_counter) {
      
     // remove mean and apodize
     if (removeMean() && apodize()) {
-      for (i_samp=0; i_samp<copyLen; i_samp++) {
-        _a[i_samp]= (_a[i_samp]-mean)*_w[i_samp];
+      for (i_samp = 0; i_samp < copyLen; ++i_samp) {
+        _a[i_samp]= (_a[i_samp] - mean)*_w[i_samp];
       }
     } else if (removeMean()) {
-      for (i_samp=0; i_samp<copyLen; i_samp++) {
+      for (i_samp = 0; i_samp < copyLen; ++i_samp) {
         _a[i_samp] -= mean;
       }
     } else if (apodize()) {
-      for (i_samp=0; i_samp<copyLen; i_samp++) {
+      for (i_samp = 0; i_samp < copyLen; ++i_samp) {
         _a[i_samp] *= _w[i_samp];
       }
     }
 
     nf += copyLen;
 
-
-    for (;i_samp < _ALen; i_samp++) {
-      _a[i_samp] = 0.0;
+    if (i_samp < _ALen) {
+      memset(&_a[i_samp], 0, sizeof(double)*(_ALen - i_samp));
     }
 
     // fft a
     rdft(_ALen, 1, _a);
     psd[0] += _a[0]*_a[0];
     psd[_PSDLen-1] += _a[1]*_a[1];
-    for (i_samp=1; i_samp<_PSDLen-1; i_samp++) {
-      psd[i_samp]+= cabs2(_a[i_samp*2], _a[i_samp*2+1]);
+    for (i_samp = 1; i_samp < _PSDLen - 1; ++i_samp) {
+      psd[i_samp] += cabs2(_a[i_samp*2], _a[i_samp*2 + 1]);
     }
   }
 
@@ -334,7 +412,7 @@ KstObject::UpdateType KstPSD::update(int update_counter) {
   _last_n_new = 0;
 
   nf = 1.0/(double(_Freq)*double(nf/2.0));
-  for ( i_samp = 0; i_samp<_PSDLen; i_samp++ ) {
+  for (i_samp = 0; i_samp < _PSDLen; ++i_samp) {
     psd[i_samp] = sqrt(psd[i_samp]*nf);
   }
 
@@ -363,6 +441,8 @@ void KstPSD::save(QTextStream &ts, const QString& indent) {
   ts << l2 << "<fftLen>" << _Len << "</fftLen>" << endl;
   ts << l2 << "<removeMean>" << _RemoveMean << "</removeMean>" << endl;
   ts << l2 << "<apodize>" << _Apodize << "</apodize>" << endl;
+  ts << l2 << "<apodizefxn>" << _apodizeFxn << "</apodizefxn>" << endl;
+  ts << l2 << "<gaussiansigma>" << _gaussianSigma << "</gaussiansigma>" << endl;
   ts << l2 << "<VUnits>" << _vUnits << "</VUnits>" << endl;
   ts << l2 << "<RUnits>" << _rUnits << "</RUnits>" << endl;
   ts << indent << "</psdobject>" << endl;
@@ -491,7 +571,7 @@ QString KstPSD::propertyString() const {
 
 
 void KstPSD::_showDialog() {
-  KstDialogs::showPSDDialog(tagName());
+  KstDialogs::self()->showPSDDialog(tagName());
 }
 
 
@@ -514,6 +594,35 @@ void KstPSD::setRUnits(const QString& units) {
   _rUnits = units;
 }
 
+int KstPSD::apodizeFxn() const {
+  return _apodizeFxn;
+}
 
+void KstPSD::setApodizeFxn(int in_apodizeFxn) {
+  setDirty();
+  _apodizeFxn = in_apodizeFxn;
+}
+
+double KstPSD::gaussianSigma() const {
+  return _gaussianSigma;
+}
+
+void KstPSD::setGaussianSigma(double in_gaussianSigma) {
+  setDirty();
+  _gaussianSigma = in_gaussianSigma;
+}
+
+ 
+KstDataObjectPtr KstPSD::makeDuplicate(KstDataObjectDataObjectMap& duplicatedMap) {
+  QString name(tagName() + '\'');
+  while (KstData::self()->dataTagNameNotUnique(name, false)) {
+    name += '\'';
+  }
+  KstPSDPtr psd = new KstPSD(name, _inputVectors[INVECTOR], _Freq,
+                             _Average, _Len, _Apodize, _RemoveMean, _vUnits, _rUnits, 
+                             _apodizeFxn, _gaussianSigma);
+  duplicatedMap.insert(this, KstDataObjectPtr(psd));    
+  return KstDataObjectPtr(psd);
+}
 
 // vim: ts=2 sw=2 et
