@@ -21,6 +21,7 @@
 #include "kstvector.h"
 
 #include "kstdatasource.h"
+#include "kst_export.h"
 
 namespace Equation {
   class Node;
@@ -29,7 +30,8 @@ namespace Equation {
 /**A class for handling data vectors for kst.
  *@author cbn
  */
-class KstRVector : public KstVector {
+   
+class KST_EXPORT KstRVector : public KstVector {
 public:
   /** Create an RVECTOR */
   KstRVector(KstDataSourcePtr file, const QString &field, const QString &tag,
@@ -110,6 +112,9 @@ public:
 
   /** Read from end */
   void setFromEnd();
+  
+  // make a copy
+  KstSharedPtr<KstRVector> makeDuplicate() const;
 
 private:
   KstObject::UpdateType doUpdate(bool force = false);
@@ -117,11 +122,11 @@ private:
   bool _dirty; // different from the KstObject dirty flag
 
   /** Common contructor for an RVector */
-  void  commonRVConstructor(KstDataSourcePtr file,
-			    const QString &field,
-                            int f0, int n,
-			    int skip, bool in_doSkip,
-                            bool in_doAve);
+  void commonRVConstructor(KstDataSourcePtr file,
+      const QString &field,
+      int f0, int n,
+      int skip, bool in_doSkip,
+      bool in_doAve);
 
   /** Samples Per Frame */
   int SPF;
@@ -166,3 +171,4 @@ typedef KstSharedPtr<KstRVector> KstRVectorPtr;
 typedef KstObjectList<KstRVectorPtr> KstRVectorList;
 
 #endif
+// vim: ts=2 sw=2 et
