@@ -20,6 +20,7 @@
 
 #include <qcolor.h>
 #include <qdom.h>
+#include <qguardedptr.h>
 
 #include <kdeversion.h>
 #if KDE_VERSION >= KDE_MAKE_VERSION(3,2,0)
@@ -28,21 +29,19 @@
 
 #include "kstobject.h"
 #include "kstalignment.h"
+#include "kstpainter.h"
+#include "kst_export.h"
 
 class KPopupMenu;
+class KstGfxMouseHandler;
 class KstViewObject;
 class KstViewWidget;
+class KstTopLevelView;
 typedef KstSharedPtr<KstViewObject> KstViewObjectPtr;
 typedef KstObjectList<KstViewObjectPtr> KstViewObjectList;
 typedef KstViewObjectPtr (*KstViewObjectFactoryMethod)();
-
-// KstPaintType - hints to paint to allow optimizations
-// P_PAINT: Neither 'data' nor 'plot' needs to change
-// P_PLOT: data didn't change
-/* FIXME: Define P_PLOT better, and do we even need P_ZOOM, P_PLOT and P_PAINT? 
-          They should all do the same thing.
-*/
-enum KstPaintType { P_PAINT, P_PLOT, P_ZOOM, P_PRINT, P_EXPORT };
+typedef KstGfxMouseHandler* (*KstHandlerFactoryMethod)();
+typedef KstSharedPtr<KstTopLevelView> KstTopLevelViewPtr;
 
 struct KstAspectRatio {
   KstAspectRatio() : x(0.0), y(0.0), w(0.0), h(0.0) {}
@@ -57,7 +56,7 @@ struct KstAspectRatio {
   }
 };
 
-class KstViewObject : public KstObject {
+class KST_EXPORT KstViewObject : public KstObject {
   Q_OBJECT
   protected:
     KstViewObject(const QString& type);
@@ -67,10 +66,14 @@ class KstViewObject : public KstObject {
     virtual ~KstViewObject();
 
     virtual UpdateType update(int = -1);
+    // FIXME: bring this down to 1 function
     virtual void save(QTextStream& ts, const QString& indent = QString::null);
-    virtual void saveTag(QTextStream& ts, const QString& indent = QString::null);
+    virtual void saveAttributes(QTextStream& ts, const QString& indent = QString::null);
     virtual void loadChildren(const QDomElement& e);
 
+    virtual void invalidateClipRegion();
+    virtual QRegion clipRegion();
+
     // If cols <= 0, optimal is chosen automatically
     virtual void cleanup(int cols = -1);
     virtual int columns() const;
@@ -79,22 +82,38 @@ class KstViewObject : public KstObject {
     virtual void setOnGrid(bool on_grid);
 
     virtual void resize(const QSize& size);
+    // FIXME: this stuff has got to go.
     virtual void resizeForPrint(const QSize& size);
     virtual void revertForPrint();
     virtual void resizeFromAspect(double x, double y, double w, double h);
     virtual QSize size() const;
-    virtual void internalAlignment(KstPaintType type, QPainter& p, QRect& plotRegion);
+    virtual void internalAlignment(KstPainter& p, QRect& plotRegion);
     virtual QPoint position() const;
     virtual const QRect& geometry() const;
+    const KstAspectRatio& aspectRatio() const;
+    virtual QRect surroundingGeometry() const;
+    // This is, by definition, the contents not including decoration. (borders
+    // etc)
     virtual QRect contentsRect() const;
     virtual void setContentsRect(const QRect& rect);
+    // This is, by definition, the meaningful internal contents.  It is often
+    // the contentsRect, but not always.  It is used to position things like
+    // arrows relative to the meaningful contents of the object.
+    virtual QRect dataRect() const;
+    virtual void setDataRect(const QRect& rect);
+
+    // with cleanup, use this factor to decide if more space should be used
+    // in the row.  1.0 means that the dataRect = geometry.  >1 means dataRect < geometry.
+    virtual double verticalSizeFactor();
+    virtual double horizontalSizeFactor();
+    
     virtual void move(const QPoint& to);
 
     // Draw a focus highlight
     virtual void setFocus(bool focus);
     virtual bool focused() const;
     virtual void setHasFocus(bool hasFocus);
-    virtual void removeFocus(QPainter& p);
+    virtual void removeFocus(KstPainter& p);
 
     virtual void appendChild(KstViewObjectPtr obj, bool keepAspect = false);
     virtual void prependChild(KstViewObjectPtr obj, bool keepAspect = false);
@@ -104,18 +123,40 @@ class KstViewObject : public KstObject {
     const KstViewObjectList& children() const;
     KstViewObjectList& children();
 
+    bool objectDirty() const; // true if this object or a child is dirty
+    bool fallThroughTransparency() const;
+    bool isResizable() const;
+    
+    virtual QWidget *configWidget();
+
     // can't be const due to KstViewObjectPtr?
-    KstViewObjectPtr findChild(const QPoint& pos);
+    // I don't like this method at all, but for now it makes things work.  Maybe
+    // split it into two methods eventually.  The bool is for toplevelview so
+    // that it can do resize rect on transparent objects
+    KstViewObjectPtr findChild(const QPoint& pos, bool borderForTransparent = false);
+    KstViewObjectPtr findDeepestChild(const QPoint& pos, bool borderForTransparent = false);
     KstViewObjectPtr findChild(const QString& name, bool recursive = true);
-    bool contains(KstViewObjectPtr child);
+    bool contains(const KstViewObjectPtr child) const;
     template<class T> KstObjectList<KstSharedPtr<T> > findChildrenType(bool recursive = false);
 
+    // Finds a container that can hold this rect.  Only searches objects that
+    // are containers.
+    KstViewObjectPtr findChild(const QRect& rect);
+    KstViewObjectPtr findDeepestChild(const QRect& rect);
+
     virtual void setBackgroundColor(const QColor& color);
     virtual QColor backgroundColor() const;
 
     virtual void setForegroundColor(const QColor& color);
     virtual QColor foregroundColor() const;
 
+    // One day maybe we'll even have translucency available!
+    virtual void setTransparent(bool transparent);
+    virtual bool transparent() const;
+
+    virtual void setFollowsFlow(bool follow);
+    virtual bool followsFlow() const;
+
     virtual void updateSelection(const QRect& region);
 
     void recursively(void (KstViewObject::*)(), bool self = false);
@@ -146,8 +187,8 @@ class KstViewObject : public KstObject {
                            CopyTo =       0x4000
                         };
 
-    virtual void drawFocusRect(QPainter& p);
-    virtual void drawSelectRect(QPainter& p);
+    virtual void drawFocusRect(KstPainter& p);
+    virtual void drawSelectRect(KstPainter& p);
 
     virtual void setMaximized(bool maximized);
     bool maximized() const;
@@ -183,9 +224,36 @@ class KstViewObject : public KstObject {
 
     void setMinimumSize(const QSize& sz);
     const QSize& minimumSize() const;
+    
+    // returns the mouse direction for this position on this view object
+    enum Direction {NONE = 0, 
+                    UP = 1, 
+                    DOWN = 2, 
+                    LEFT = 4, 
+                    RIGHT = 8, 
+                    CENTEREDRESIZE = 16,
+                    ENDPOINT = 32};
+    virtual signed int directionFor(const QPoint& pos);
+    
+    // returns instructions for the edit dialog of this view object
+    // given a property name
+    // FIXME: Don't use QMap.  We need to preserve order here.
+    virtual QMap<QString, QVariant > widgetHints(const QString& propertyName) const;
+
+    // draws a minimalistic shadow outline of this object at the given position
+    virtual void drawShadow(KstPainter& p, const QPoint& pos);
+
+    // handle custom widget, if any: is called by KstEditViewObjectDialogI
+    virtual bool fillConfigWidget(QWidget *w, bool isNew) const;
+    virtual bool readConfigWidget(QWidget *w);
+
+    virtual KstGfxMouseHandler *createHandler();
 
   public slots:
-    virtual void paint(KstPaintType type, QPainter& p);
+    void paint(KstPainter& p, const QRegion& bounds);
+    virtual void paintSelf(KstPainter& p, const QRegion& bounds);
+    virtual void updateSelf();
+    virtual void paintUpdate();
     virtual void updateFromAspect();
     virtual void updateAspectSize();
     virtual void updateAspectPos();
@@ -193,6 +261,7 @@ class KstViewObject : public KstObject {
     virtual void zoomToggle();
     virtual void copyObject();
     virtual void copyObjectQuietly(KstViewObject& parent, const QString& name = QString::null) const;
+    virtual bool showDialog(KstTopLevelViewPtr invoker, bool isNew = false);
 
   protected slots:
     virtual void parentResized();
@@ -210,12 +279,22 @@ class KstViewObject : public KstObject {
     virtual void copyTo(int);
     virtual void rename();
     virtual void pauseToggle();
+    virtual void edit();
 
   protected:
+    friend class KstTopLevelView;
+    friend class KstEditViewObjectDialogI;
+    void setDialogLock(bool lock);
+    bool dialogLocked() const;
     virtual KstViewObjectFactoryMethod factory() const;
+    virtual KstHandlerFactoryMethod handlerFactory() const;
     virtual void writeBinary(QDataStream& str);
     virtual void readBinary(QDataStream& str);
-
+    // id=0 v1.2.0 setDirty() was called.  data == 0 means false, else true
+    virtual void virtual_hook(int id, void *data);
+    
+    KstObject::UpdateType updateChildren(int counter);
+    
     KstViewObjectList _children;
     QRect _geom;
     QRect _geomOld;
@@ -226,19 +305,21 @@ class KstViewObject : public KstObject {
     bool _onGrid : 1;
     bool _prevOnGrid : 1;
     bool _maximized : 1;
+    bool _transparent : 1;
+    bool _followsFlow : 1;
+    bool _dialogLock : 1;
+    bool _container : 1;
+    bool _fallThroughTransparency : 1;
+    bool _isResizable : 1;
     int _columns : 6; // "64 columns ought to be enough for anyone"
-    KstViewObject *_parent; // danger!!
-    //KstViewObjectPtr _parent; // FIXME: this is bad and should be removed ASAP
-                              //        It was introduced as a temporary hack
-                              //        but is no longer needed as events now
-                              //        pass the view pointer along.  Also use
-                              //        of this member indicates a bug since
-                              //        objects can have multiple parents.
+    QGuardedPtr<KstViewObject> _topObjectForMenu;
+    QGuardedPtr<KstViewObject> _parent; // danger!!
     Q_UINT32 _standardActions, _layoutActions;
     KstAspectRatio _aspect;
     KstAspectRatio _aspectOldZoomedObject;
     QString _type;
     QRegion _lastClipRegion; // a sort of cache - might not always work
+    QRegion _clipMask; // The mask of this object
     QMap<int, QString> _moveToMap;
     QMap<int, QString> _copyToMap;
     QSize _minimumSize;
