@@ -16,11 +16,12 @@
  ***************************************************************************/
 
 // include files for Qt
+#include <qfile.h>
+#include <qpainter.h>
 #include <qpaintdevicemetrics.h>
 #include <qstylesheet.h>
 
 // include files for KDE
-#include <kdebug.h>
 #include <kmessagebox.h>
 #include <kprinter.h>
 
@@ -91,6 +92,22 @@ void KstViewWindow::commonConstructor() {
   vb->setAutoAdd(true);
 }
 
+
+void KstViewWindow::rename() {
+  KstApp::inst()->renameWindow(this);
+}
+
+
+void KstViewWindow::moveTabLeft() {
+  KstApp::inst()->moveTabLeft(this);
+}
+
+
+void KstViewWindow::moveTabRight() {
+  KstApp::inst()->moveTabRight(this);
+}
+
+
 void KstViewWindow::updateActions() {
 }
 
@@ -143,39 +160,38 @@ void KstViewWindow::immediatePrintToFile(const QString &filename) {
   printer.setOutputToFile(true);
   printer.setOutputFileName(filename);
 
-  QPainter paint(&printer);
+  KstPainter p(KstPainter::P_PRINT);
+  p.begin(&printer);
   QPaintDeviceMetrics metrics(&printer);
-  QSize size(metrics.width(), metrics.height());
+  const QSize size(metrics.width(), metrics.height());
 
   view()->resizeForPrint(size);
-  view()->paint(P_PRINT, paint);
+  view()->paint(p, QRegion());
   view()->revertForPrint();
+  p.end();
 }
 
 
 void KstViewWindow::immediatePrintToPng(QDataStream* dataStream, const QSize& size, const QString &format) {
-  QPixmap pixmap(size);
-  QSize sizeOld;
-
   if (!view()->children().isEmpty()) {
-    QPainter paint;
-
+    QPixmap pixmap(size);
+    KstPainter paint(KstPainter::P_EXPORT);
     if (paint.begin(&pixmap)) {
-      sizeOld = view()->size();
+      const QSize sizeOld(view()->size());
       view()->resizeForPrint(size);
-      view()->paint(P_EXPORT, paint);
+      view()->paint(paint, QRegion());
 #if QT_VERSION >= 0x030200
       pixmap.save(dataStream->device(), format.latin1());
 #else
       Q_UNUSED(format)
-      
+
       QByteArray bytes;
       QDataStream tempStream(bytes, IO_ReadWrite);
-      
+
       tempStream << pixmap;
       if (bytes.count() > 4) {
-        dataStream->writeRawBytes(bytes.data()+4, bytes.count()-4); 
-      }     
+        dataStream->writeRawBytes(bytes.data()+4, bytes.count()-4);
+      }
 #endif
       view()->revertForPrint();
       paint.end();
@@ -186,13 +202,14 @@ void KstViewWindow::immediatePrintToPng(QDataStream* dataStream, const QSize& si
 
 void KstViewWindow::immediatePrintToPng(const QString &filename, const QSize& size, const QString &format) {
   if (!view()->children().isEmpty()) {
-    QPainter paint;
+    KstPainter paint(KstPainter::P_EXPORT);
     QPixmap pixmap(size);
 
     if (paint.begin(&pixmap)) {
       QString dotFormat = QString(".") + format;
-      int iPos = filename.findRev(dotFormat, -1, FALSE);
       QString filenameNew;
+      int iPos = filename.findRev(dotFormat, -1, false);
+
       if (iPos != -1 && iPos == (int)(filename.length() - dotFormat.length())) {
         filenameNew = filename;
       } else {
@@ -200,7 +217,7 @@ void KstViewWindow::immediatePrintToPng(const QString &filename, const QSize& si
       }
 
       view()->resizeForPrint(size);
-      view()->paint(P_EXPORT, paint);
+      view()->paint(paint, QRegion());
       if (!pixmap.save(filenameNew, format.latin1())) {
         KstDebug::self()->log(i18n("Failed to save image to %1").arg(filename), KstDebug::Warning);
       }
@@ -211,9 +228,77 @@ void KstViewWindow::immediatePrintToPng(const QString &filename, const QSize& si
 }
 
 
+// FIXME: don't write to disk like this.  write to a pipe or at least use
+// KTempFile
+void KstViewWindow::immediatePrintToEps(const QString &filename, const QSize& size) {
+  if (!view()->children().isEmpty()) {
+    QString filenameNew;
+    QString filenameNewEps;
+
+    {
+      QPrinter printer;
+      QString dotFormat = QString(".eps");
+      int iPos = filename.findRev(dotFormat, -1, false);
+
+      if (iPos != -1 && iPos == (int)(filename.length() - dotFormat.length())) {
+        filenameNewEps = filename;
+      } else {
+        filenameNewEps = filename + dotFormat;
+      }
+      filenameNew = filenameNewEps + ".ps";
+
+      printer.setPageSize(QPrinter::Letter);
+      printer.setOrientation(QPrinter::Landscape);
+      printer.setOutputToFile(true);
+      printer.setOutputFileName(filenameNew);
+
+      KstPainter paint(KstPainter::P_PRINT);
+      paint.begin(&printer);
+      QPaintDeviceMetrics metrics(&printer);
+
+      view()->resizeForPrint(size);
+      view()->paint(paint, QRegion());
+      view()->revertForPrint();
+      paint.end();
+    }
+
+    //
+    // now try to open the ps file and convert it to an eps...
+    //
+    QFile filePS(filenameNew);
+    QFile fileEPS(filenameNewEps);
+    QString line;
+
+    if (filePS.open(IO_ReadOnly)) {
+      if (fileEPS.open(IO_WriteOnly | IO_Truncate)) {
+        QTextStream streamPS(&filePS);
+        QTextStream streamEPS(&fileEPS);
+
+        line = streamPS.readLine();
+        if (!line.isNull()) {
+          if (line.left(11) == "%!PS-Adobe-") {
+            //
+            // we have a ps file, so do the conversion...
+            //
+            streamEPS << "%!PS-Adobe-2.0 EPSF-2.0\n";
+            while (!streamPS.atEnd()) {
+              line = streamPS.readLine();
+              streamEPS << line << "\n";
+            }
+          }
+        }
+        fileEPS.close();
+      }
+      filePS.close();
+      filePS.remove();
+    }
+  }
+}
+
+
 void KstViewWindow::save(QTextStream& ts, const QString& indent) {
-  QRect restoreGeom = restoreGeometry();
-  QRect internalGeom = internalGeometry();
+  const QRect restoreGeom(restoreGeometry());
+  const QRect internalGeom(internalGeometry());
 
   ts << indent << "<tag>" << QStyleSheet::escape(caption()) << "</tag>" << endl;
 
@@ -234,7 +319,7 @@ void KstViewWindow::save(QTextStream& ts, const QString& indent) {
     ts << indent << "<maximized/>" << endl;
   }
 
-  view()->save(ts, indent + "  ");
+  view()->save(ts, indent);
 }
 
 
@@ -242,25 +327,21 @@ KstTopLevelViewPtr KstViewWindow::view() const {
   return _view;
 }
 
-void KstViewWindow::setCaption(const QString& szCaption) {
-  KMdiChildView::setCaption(szCaption);
-  _view->setTagName(szCaption);
+
+void KstViewWindow::setCaption(const QString& caption) {
+  KMdiChildView::setCaption(caption);
+  _view->setTagName(caption);
 }
 
+
 void KstViewWindow::closeEvent(QCloseEvent *e) {
-  if (KstSettings::globalSettings()->promptWindowClose) {
-    if (view()->children().size()) {
-      if (KMessageBox::warningYesNo(0L, i18n("Are you sure you want to close window '%1'?\nClosing a window deletes all plots in the window.").arg(caption())) == KMessageBox::Yes) {
-        KMdiChildView::closeEvent(e);
-      } else {
-        e->ignore();
-      }
-    } else {
-      KMdiChildView::closeEvent(e);
+  if (KstSettings::globalSettings()->promptWindowClose && !view()->children().isEmpty()) {
+    if (KMessageBox::warningYesNo(this, i18n("Are you sure you want to close window '%1'?\nClosing a window deletes all plots in the window.").arg(caption())) != KMessageBox::Yes) {
+      e->ignore();
+      return;
     }
-  } else {
-    KMdiChildView::closeEvent(e);
   }
+  KMdiChildView::closeEvent(e);
 }
 
 #include "kstviewwindow.moc"
