@@ -7,7 +7,6 @@
 ** place of a destructor.
 *****************************************************************************/
 
-
 void MatrixSelector::allowNewMatrices( bool allowed )
 {
     _newMatrix->setEnabled(allowed);
@@ -37,13 +36,19 @@ void MatrixSelector::update()
     if (_provideNoneMatrix) {
 	_matrix->insertItem("<None>");
     }
-    KstMatrixList matrices = kstObjectSubList<KstDataObject, KstMatrix>(KST::dataObjectList);
-    for (KstMatrixList::Iterator i = matrices.begin(); i != matrices.end(); ++i) {
-	_matrix->insertItem((*i)->tagName());
-	if (!found && (*i)->tagName() == prev) {
+
+    KstMatrixList matrices = KST::matrixList;
+    KST::matrixList.lock().readLock();
+    for (KstMatrixList::ConstIterator i = matrices.begin(); i != matrices.end(); ++i) {
+	(*i)->readLock();
+	QString tag = (*i)->tagName();
+	(*i)->readUnlock();
+	_matrix->insertItem(tag);
+	if (!found && tag == prev) {
 	    found = true;
 	}
     }
+    KST::matrixList.lock().readUnlock();
     if (found) {
 	_matrix->setCurrentText(prev);
     }
@@ -62,19 +67,13 @@ void MatrixSelector::init()
 
 void MatrixSelector::createNewMatrix()
 {
-    KstMatrixDialogI *ad = new KstMatrixDialogI(this, "matrix dialog");
-    connect(ad, SIGNAL(matrixCreated(KstMatrixPtr)), this, SLOT(newMatrixCreated(KstMatrixPtr)));
-    connect(ad, SIGNAL(matrixCreated(KstMatrixPtr)), this, SLOT(setSelection(KstMatrixPtr)));
-    connect(ad, SIGNAL(modified()), this, SLOT(update()));
-    ad->show_New();
-    ad->exec();
-    delete ad;
+    KstDialogs::self()->newMatrixDialog(this, SLOT(newMatrixCreated(KstMatrixPtr)), SLOT(setSelection(KstMatrixPtr)), SLOT(update()));
 }
 
 
 void MatrixSelector::selectionWatcher( const QString & tag )
 {
-    QString label = "["+tag+"]";
+    QString label = "[" + tag + "]";
     emit selectionChangedLabel(label);
     setEdit(tag);
 }
@@ -101,7 +100,9 @@ void MatrixSelector::setSelection( const QString & tag )
 
 void MatrixSelector::newMatrixCreated( KstMatrixPtr v )
 {
+    v->readLock();
     QString name = v->tagName();
+    v->readUnlock();
     v = 0L; // deref
     emit newMatrixCreated(name);
 }
@@ -109,7 +110,9 @@ void MatrixSelector::newMatrixCreated( KstMatrixPtr v )
 
 void MatrixSelector::setSelection( KstMatrixPtr v )
 {
+    v->readLock();
     setSelection(v->tagName());
+    v->readUnlock();
 }
 
 
@@ -124,14 +127,15 @@ void MatrixSelector::provideNoneMatrix( bool provide )
 
 void MatrixSelector::editMatrix()
 {
-    KstMatrixDialogI::globalInstance()->show_Edit(_matrix->currentText());
+    KstDialogs::self()->showMatrixDialog(_matrix->currentText());
 }
 
 
-void MatrixSelector::setEdit(QString tag)
+void MatrixSelector::setEdit(const QString& tag)
 {
-    KstMatrixList matrices = kstObjectSubList<KstDataObject, KstMatrix>(KST::dataObjectList);
-    _editMatrix->setEnabled(matrices.findTag(tag) != matrices.end());
+    KST::matrixList.lock().readLock();
+    _editMatrix->setEnabled(KST::matrixList.findTag(tag) != KST::matrixList.end());
+    KST::matrixList.lock().readUnlock();
 }
 
 // vim: ts=8 sw=4 noet
