@@ -26,7 +26,7 @@ void PluginManager::selectionChanged( QListViewItem *item )
 
 void PluginManager::install()
 {
-    KURL xmlfile = KFileDialog::getOpenURL(QString::null, "*.xml", this, i18n("Please Select Plugin to Install"));
+    KURL xmlfile = KFileDialog::getOpenURL(QString::null, "*.xml", this, i18n("Select Plugin to Install"));
 
     if (xmlfile.isEmpty()) {
 	return;
